# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from copy import deepcopy

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list

from ansible_collections.community.aws.plugins.module_utils.base import BaseResourceManager
from ansible_collections.community.aws.plugins.module_utils.base import BaseWaiterFactory
from ansible_collections.community.aws.plugins.module_utils.base import Boto3Mixin


class Ec2WaiterFactory(BaseWaiterFactory):
    def __init__(self, module):
        # the AWSRetry wrapper doesn't support the wait functions (there's no
        # public call we can cleanly wrap)
        client = module.client("ec2")
        super(Ec2WaiterFactory, self).__init__(module, client)

    @property
    def _waiter_model_data(self):
        data = super(Ec2WaiterFactory, self)._waiter_model_data
        return data


class Ec2Boto3Mixin(Boto3Mixin):
    @AWSRetry.jittered_backoff()
    def _paginated_describe_subnets(self, **params):
        paginator = self.client.get_paginator("describe_subnets")
        return paginator.paginate(**params).build_full_result()

    @Boto3Mixin.aws_error_handler("describe subnets")
    def _describe_subnets(self, **params):
        try:
            result = self._paginated_describe_subnets(**params)
        except is_boto3_error_code("SubnetID.NotFound"):
            return None
        return result.get("Subnets", None)


class BaseEc2Manager(Ec2Boto3Mixin, BaseResourceManager):
    resource_id = None
    TAG_RESOURCE_TYPE = None
    # This can be overridden by a subclass *if* 'Tags' isn't returned as a part of
    # the standard Resource description
    TAGS_ON_RESOURCE = True
    # If the resource supports using "TagSpecifications" on creation we can
    TAGS_ON_CREATE = "TagSpecifications"

    def __init__(self, module, resource_id=None):
        r"""
        Parameters:
            module (AnsibleAWSModule): An Ansible module.
        """
        super(BaseEc2Manager, self).__init__(module)
        self.client = self._create_client()
        self._tagging_updates = dict()
        self.resource_id = resource_id

        # Name parameter is unique (by region) and can not be modified.
        if self.resource_id:
            resource = deepcopy(self.get_resource())
            self.original_resource = resource

    def _flush_update(self):
        changed = False
        changed |= self._do_tagging()
        changed |= super(BaseEc2Manager, self)._flush_update()
        return changed

    @Boto3Mixin.aws_error_handler("connect to AWS")
    def _create_client(self, client_name="ec2"):
        client = self.module.client(client_name, retry_decorator=AWSRetry.jittered_backoff())
        return client

    @Boto3Mixin.aws_error_handler("set tags on resource")
    def _add_tags(self, **params):
        self.client.create_tags(aws_retry=True, **params)
        return True

    @Boto3Mixin.aws_error_handler("unset tags on resource")
    def _remove_tags(self, **params):
        self.client.delete_tags(aws_retry=True, **params)
        return True

    @AWSRetry.jittered_backoff()
    def _paginated_describe_tags(self, **params):
        paginator = self.client.get_paginator("describe_tags")
        return paginator.paginate(**params).build_full_result()

    @Boto3Mixin.aws_error_handler("list tags on resource")
    def _describe_tags(self, resource_id=None):
        if not resource_id:
            resource_id = self.resource_id
        filters = ansible_dict_to_boto3_filter_list({"resource-id": resource_id})
        tags = self._paginated_describe_tags(Filters=filters)
        return tags

    def _get_tags(self, resource_id=None):
        if resource_id is None:
            resource_id = self.resource_id
        # If the Tags are available from the resource, then use them
        if self.TAGS_ON_RESOURCE:
            tags = self._preupdate_resource.get("Tags", [])
        # Otherwise we'll have to look them up
        else:
            tags = self._describe_tags(resource_id=resource_id)
        return boto3_tag_list_to_ansible_dict(tags)

    def _do_tagging(self):
        changed = False
        tags_to_add = self._tagging_updates.get("add")
        tags_to_remove = self._tagging_updates.get("remove")

        if tags_to_add:
            changed = True
            tags = ansible_dict_to_boto3_tag_list(tags_to_add)
            if not self.module.check_mode:
                self._add_tags(Resources=[self.resource_id], Tags=tags)
        if tags_to_remove:
            changed = True
            if not self.module.check_mode:
                tag_list = [dict(Key=tagkey) for tagkey in tags_to_remove]
                self._remove_tags(Resources=[self.resource_id], Tags=tag_list)

        return changed

    def _merge_resource_changes(self, filter_immutable=True, creation=False):
        resource = super(BaseEc2Manager, self)._merge_resource_changes(
            filter_immutable=filter_immutable, creation=creation
        )

        if creation:
            if not self.TAGS_ON_CREATE:
                resource.pop("Tags", None)
            elif self.TAGS_ON_CREATE == "TagSpecifications":
                tags = boto3_tag_list_to_ansible_dict(resource.pop("Tags", []))
                tag_specs = boto3_tag_specifications(tags, types=[self.TAG_RESOURCE_TYPE])
                if tag_specs:
                    resource["TagSpecifications"] = tag_specs

        return resource

    def set_tags(self, tags, purge_tags):
        if tags is None:
            return False
        changed = False

        # Tags are returned as a part of the resource, but have to be updated
        # via dedicated tagging methods
        current_tags = self._get_tags()

        # So that diff works in check mode we need to know the full target state
        if purge_tags:
            desired_tags = deepcopy(tags)
        else:
            desired_tags = deepcopy(current_tags)
            desired_tags.update(tags)

        tags_to_add, tags_to_remove = compare_aws_tags(current_tags, tags, purge_tags)

        if tags_to_add:
            self._tagging_updates["add"] = tags_to_add
            changed = True
        if tags_to_remove:
            self._tagging_updates["remove"] = tags_to_remove
            changed = True

        if changed:
            # Tags are a stored as a list, but treated like a list, the
            # simplisic '==' in _set_resource_value doesn't do the comparison
            # properly
            return self._set_resource_value("Tags", ansible_dict_to_boto3_tag_list(desired_tags))

        return False



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: opensearch_info
short_description: obtain information about one or more OpenSearch or ElasticSearch domain
description:
  - Obtain information about one Amazon OpenSearch Service domain.
version_added: 4.0.0
author:
  - "Sebastien Rosset (@sebastien-rosset)"
options:
  domain_name:
    description:
      - The name of the Amazon OpenSearch/ElasticSearch Service domain.
    required: false
    type: str
  tags:
    description:
      - >
        A dict of tags that are used to filter OpenSearch domains that match
        all tag key, value pairs.
    required: false
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Get information about an OpenSearch domain instance
  community.aws.opensearch_info:
    domain_name: my-search-cluster
  register: new_cluster_info

- name: Get all OpenSearch instances
  community.aws.opensearch_info:

- name: Get all OpenSearch instances that have the specified Key, Value tags
  community.aws.opensearch_info:
    tags:
      Applications: search
      Environment: Development
"""

RETURN = r"""
instances:
  description: List of OpenSearch domain instances
  returned: always
  type: complex
  contains:
    domain_status:
      description: The current status of the OpenSearch domain.
      returned: always
      type: complex
      contains:
        arn:
          description: The ARN of the OpenSearch domain.
          returned: always
          type: str
        domain_id:
          description: The unique identifier for the OpenSearch domain.
          returned: always
          type: str
        domain_name:
          description: The name of the OpenSearch domain.
          returned: always
          type: str
        created:
          description:
            - >
              The domain creation status. True if the creation of a domain is complete.
              False if domain creation is still in progress.
          returned: always
          type: bool
        deleted:
          description:
            - >
              The domain deletion status.
              True if a delete request has been received for the domain but resource cleanup is still in progress.
              False if the domain has not been deleted.
              Once domain deletion is complete, the status of the domain is no longer returned.
          returned: always
          type: bool
        endpoint:
          description: The domain endpoint that you use to submit index and search requests.
          returned: always
          type: str
        endpoints:
          description:
            - >
              Map containing the domain endpoints used to submit index and search requests.
            - >
              When you create a domain attached to a VPC domain, this propery contains
              the DNS endpoint to which service requests are submitted.
            - >
              If you query the opensearch_info immediately after creating the OpenSearch cluster,
              the VPC endpoint may not be returned. It may take several minutes until the
              endpoints is available.
          type: dict
        processing:
          description:
            - >
              The status of the domain configuration.
              True if Amazon OpenSearch Service is processing configuration changes.
              False if the configuration is active.
          returned: always
          type: bool
        upgrade_processing:
          description: true if a domain upgrade operation is in progress.
          returned: always
          type: bool
        engine_version:
          description: The version of the OpenSearch domain.
          returned: always
          type: str
          sample: OpenSearch_1.1
        cluster_config:
          description:
            - Parameters for the cluster configuration of an OpenSearch Service domain.
          type: complex
          contains:
            instance_type:
              description:
                - Type of the instances to use for the domain.
              type: str
            instance_count:
              description:
                - Number of instances for the domain.
              type: int
            zone_awareness:
              description:
                - A boolean value to indicate whether zone awareness is enabled.
              type: bool
            availability_zone_count:
              description:
                - >
                  An integer value to indicate the number of availability zones for a domain when zone awareness is enabled.
                  This should be equal to number of subnets if VPC endpoints is enabled.
              type: int
            dedicated_master_enabled:
              description:
                - A boolean value to indicate whether a dedicated master node is enabled.
              type: bool
            zone_awareness_enabled:
              description:
                - A boolean value to indicate whether zone awareness is enabled.
              type: bool
            zone_awareness_config:
              description:
                - The zone awareness configuration for a domain when zone awareness is enabled.
              type: complex
              contains:
                availability_zone_count:
                  description:
                    - An integer value to indicate the number of availability zones for a domain when zone awareness is enabled.
                  type: int
            dedicated_master_type:
              description:
                - The instance type for a dedicated master node.
              type: str
            dedicated_master_count:
              description:
                - Total number of dedicated master nodes, active and on standby, for the domain.
              type: int
            warm_enabled:
              description:
                - True to enable UltraWarm storage.
              type: bool
            warm_type:
              description:
                - The instance type for the OpenSearch domain's warm nodes.
              type: str
            warm_count:
              description:
                - The number of UltraWarm nodes in the domain.
              type: int
            cold_storage_options:
              description:
                - Specifies the ColdStorageOptions config for a Domain.
              type: complex
              contains:
                enabled:
                  description:
                    - True to enable cold storage. Supported on Elasticsearch 7.9 or above.
                  type: bool
        ebs_options:
          description:
            - Parameters to configure EBS-based storage for an OpenSearch Service domain.
          type: complex
          contains:
            ebs_enabled:
              description:
                - Specifies whether EBS-based storage is enabled.
              type: bool
            volume_type:
              description:
                - Specifies the volume type for EBS-based storage. "standard"|"gp2"|"io1"
              type: str
            volume_size:
              description:
                - Integer to specify the size of an EBS volume.
              type: int
            iops:
              description:
                - The IOPD for a Provisioned IOPS EBS volume (SSD).
              type: int
        vpc_options:
          description:
            - Options to specify the subnets and security groups for a VPC endpoint.
          type: complex
          contains:
            vpc_id:
              description: The VPC ID for the domain.
              type: str
            subnet_ids:
              description:
                - Specifies the subnet ids for VPC endpoint.
              type: list
              elements: str
            security_group_ids:
              description:
                - Specifies the security group ids for VPC endpoint.
              type: list
              elements: str
            availability_zones:
              description:
                - The Availability Zones for the domain..
              type: list
              elements: str
        snapshot_options:
          description:
            - Option to set time, in UTC format, of the daily automated snapshot.
          type: complex
          contains:
            automated_snapshot_start_hour:
              description:
                - >
                  Integer value from 0 to 23 specifying when the service takes a daily automated snapshot
                  of the specified Elasticsearch domain.
              type: int
        access_policies:
          description:
            - IAM access policy as a JSON-formatted string.
          type: complex
        encryption_at_rest_options:
          description:
            - Parameters to enable encryption at rest.
          type: complex
          contains:
            enabled:
              description:
                - Should data be encrypted while at rest.
              type: bool
            kms_key_id:
              description:
                - If encryption at rest enabled, this identifies the encryption key to use.
                - The value should be a KMS key ARN. It can also be the KMS key id.
              type: str
        node_to_node_encryption_options:
          description:
            - Node-to-node encryption options.
          type: complex
          contains:
            enabled:
              description:
                  - True to enable node-to-node encryption.
              type: bool
        cognito_options:
          description:
            - Parameters to configure OpenSearch Service to use Amazon Cognito authentication for OpenSearch Dashboards.
          type: complex
          contains:
            enabled:
              description:
                - The option to enable Cognito for OpenSearch Dashboards authentication.
              type: bool
            user_pool_id:
              description:
                - The Cognito user pool ID for OpenSearch Dashboards authentication.
              type: str
            identity_pool_id:
              description:
                - The Cognito identity pool ID for OpenSearch Dashboards authentication.
              type: str
            role_arn:
              description:
                - The role ARN that provides OpenSearch permissions for accessing Cognito resources.
              type: str
        domain_endpoint_options:
          description:
            - Options to specify configuration that will be applied to the domain endpoint.
          type: complex
          contains:
            enforce_https:
              description:
                - Whether only HTTPS endpoint should be enabled for the domain.
              type: bool
            tls_security_policy:
              description:
                - Specify the TLS security policy to apply to the HTTPS endpoint of the domain.
              type: str
            custom_endpoint_enabled:
              description:
                - Whether to enable a custom endpoint for the domain.
              type: bool
            custom_endpoint:
              description:
                - The fully qualified domain for your custom endpoint.
              type: str
            custom_endpoint_certificate_arn:
              description:
                - The ACM certificate ARN for your custom endpoint.
              type: str
        advanced_security_options:
          description:
            - Specifies advanced security options.
          type: complex
          contains:
            enabled:
              description:
                - True if advanced security is enabled.
                - You must enable node-to-node encryption to use advanced security options.
              type: bool
            internal_user_database_enabled:
              description:
                - True if the internal user database is enabled.
              type: bool
            master_user_options:
              description:
                - Credentials for the master user, username and password, ARN, or both.
              type: complex
              contains:
                master_user_arn:
                  description:
                    - ARN for the master user (if IAM is enabled).
                  type: str
                master_user_name:
                  description:
                    - The username of the master user, which is stored in the Amazon OpenSearch Service domain internal database.
                  type: str
                master_user_password:
                  description:
                    - The password of the master user, which is stored in the Amazon OpenSearch Service domain internal database.
                  type: str
            saml_options:
              description:
                - The SAML application configuration for the domain.
              type: complex
              contains:
                enabled:
                  description:
                    - True if SAML is enabled.
                  type: bool
                idp:
                  description:
                    - The SAML Identity Provider's information.
                  type: complex
                  contains:
                    metadata_content:
                      description:
                        - The metadata of the SAML application in XML format.
                      type: str
                    entity_id:
                      description:
                        - The unique entity ID of the application in SAML identity provider.
                      type: str
                master_user_name:
                  description:
                    - The SAML master username, which is stored in the Amazon OpenSearch Service domain internal database.
                  type: str
                master_backend_role:
                  description:
                    - The backend role that the SAML master user is mapped to.
                  type: str
                subject_key:
                  description:
                    - Element of the SAML assertion to use for username. Default is NameID.
                  type: str
                roles_key:
                  description:
                    - Element of the SAML assertion to use for backend roles. Default is roles.
                  type: str
                session_timeout_minutes:
                  description:
                    - >
                      The duration, in minutes, after which a user session becomes inactive.
                      Acceptable values are between 1 and 1440, and the default value is 60.
                  type: int
        auto_tune_options:
          description:
            - Specifies Auto-Tune options.
          type: complex
          contains:
            desired_state:
              description:
                - The Auto-Tune desired state. Valid values are ENABLED and DISABLED.
              type: str
            maintenance_schedules:
              description:
                - A list of maintenance schedules.
              type: list
              elements: dict
              contains:
                start_at:
                  description:
                    - The timestamp at which the Auto-Tune maintenance schedule starts.
                  type: str
                duration:
                  description:
                    - Specifies maintenance schedule duration, duration value and duration unit.
                  type: complex
                  contains:
                    value:
                      description:
                        - Integer to specify the value of a maintenance schedule duration.
                      type: int
                    unit:
                      description:
                        - The unit of a maintenance schedule duration. Valid value is HOURS.
                      type: str
                cron_expression_for_recurrence:
                  description:
                    - A cron expression for a recurring maintenance schedule.
                  type: str
    domain_config:
      description: The OpenSearch domain configuration
      returned: always
      type: complex
      contains:
        domain_name:
          description: The name of the OpenSearch domain.
          returned: always
          type: str
"""


try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.opensearch import get_domain_config
from ansible_collections.community.aws.plugins.module_utils.opensearch import get_domain_status


def domain_info(client, module):
    domain_name = module.params.get("domain_name")
    filter_tags = module.params.get("tags")

    domain_list = []
    if domain_name:
        domain_status = get_domain_status(client, module, domain_name)
        if domain_status:
            domain_list.append({"DomainStatus": domain_status})
    else:
        domain_summary_list = client.list_domain_names()["DomainNames"]
        for d in domain_summary_list:
            domain_status = get_domain_status(client, module, d["DomainName"])
            if domain_status:
                domain_list.append({"DomainStatus": domain_status})

    # Get the domain tags
    for domain in domain_list:
        current_domain_tags = None
        domain_arn = domain["DomainStatus"]["ARN"]
        try:
            current_domain_tags = client.list_tags(ARN=domain_arn, aws_retry=True)["TagList"]
            domain["Tags"] = boto3_tag_list_to_ansible_dict(current_domain_tags)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError):
            # This could potentially happen if a domain is deleted between the time
            # its domain status was queried and the tags were queried.
            domain["Tags"] = {}

    # Filter by tags
    if filter_tags:
        for tag_key in filter_tags:
            try:
                domain_list = [
                    c
                    for c in domain_list
                    if ("Tags" in c) and (tag_key in c["Tags"]) and (c["Tags"][tag_key] == filter_tags[tag_key])
                ]
            except (TypeError, AttributeError) as e:
                module.fail_json(msg="OpenSearch tag filtering error", exception=e)

    # Get the domain config
    for idx, domain in enumerate(domain_list):
        domain_name = domain["DomainStatus"]["DomainName"]
        (domain_config, arn) = get_domain_config(client, module, domain_name)
        if domain_config:
            domain["DomainConfig"] = domain_config
        domain_list[idx] = camel_dict_to_snake_dict(domain, ignore_list=["AdvancedOptions", "Endpoints", "Tags"])

    return dict(changed=False, domains=domain_list)


def main():
    module = AnsibleAWSModule(
        argument_spec=dict(
            domain_name=dict(required=False),
            tags=dict(type="dict", required=False),
        ),
        supports_check_mode=True,
    )

    try:
        client = module.client("opensearch", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS opensearch service")

    module.exit_json(**domain_info(client, module))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networkfirewall_rule_group_info
short_description: describe AWS Network Firewall rule groups
version_added: 4.0.0
description:
  - A module for describing AWS Network Firewall rule groups.
options:
  arn:
    description:
      - The ARN of the Network Firewall rule group.
      - At time of writing AWS does not support describing Managed Rules.
    required: false
    type: str
  name:
    description:
      - The name of the Network Firewall rule group.
    required: false
    type: str
  rule_type:
    description:
      - Indicates whether the rule group is stateless or stateful.
      - Required if I(name) is provided.
    required: false
    aliases: ['type' ]
    choices: ['stateful', 'stateless']
    type: str
  scope:
    description:
      - The scope of the request.
      - When I(scope='account') returns a description of all rule groups in the account.
      - When I(scope='managed') returns a list of available managed rule group arns.
      - By default searches only at the account scope.
    required: false
    choices: ['managed', 'account']
    type: str

author:
  - Mark Chappell (@tremble)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""

# Describe all Rule Groups in an account (excludes managed groups)
- community.aws.networkfirewall_rule_group_info: {}

# List the available Managed Rule groups (AWS doesn't support describing the
# groups)
- community.aws.networkfirewall_rule_group_info:
    scope: managed

# Describe a Rule Group by ARN
- community.aws.networkfirewall_rule_group_info:
    arn: arn:aws:network-firewall:us-east-1:123456789012:stateful-rulegroup/ExampleRuleGroup

# Describe a Rule Group by name
- community.aws.networkfirewall_rule_group_info:
    name: ExampleRuleGroup
    type: stateful
"""

RETURN = r"""
rule_list:
  description: A list of ARNs of the matching rule groups.
  type: list
  elements: str
  returned: When a rule name isn't specified

rule_groups:
  description: The details of the rule groups
  returned: success
  type: list
  elements: dict
  contains:
    rule_group:
      description: Details of the rules in the rule group
      type: dict
      returned: success
      contains:
        rule_variables:
          description: Settings that are available for use in the rules in the rule group.
          returned: When rule variables are attached to the rule group.
          type: complex
          contains:
            ip_sets:
              description: A dictionary mapping variable names to IP addresses in CIDR format.
              returned: success
              type: dict
              example: ['192.0.2.0/24']
            port_sets:
              description: A dictionary mapping variable names to ports
              returned: success
              type: dict
              example: ['42']
        stateful_rule_options:
          description: Additional options governing how Network Firewall handles stateful rules.
          returned: When the rule group is either "rules string" or "rules list" based.
          type: dict
          contains:
            rule_order:
              description: The order in which rules will be evaluated.
              returned: success
              type: str
              example: 'DEFAULT_ACTION_ORDER'
        rules_source:
          description: DEFAULT_ACTION_ORDER
          returned: success
          type: dict
          contains:
            stateful_rules:
              description: A list of dictionaries describing the rules that the rule group is comprised of.
              returned: When the rule group is "rules list" based.
              type: list
              elements: dict
              contains:
                action:
                  description: What action to perform when a flow matches the rule criteria.
                  returned: success
                  type: str
                  example: 'PASS'
                header:
                  description: A description of the criteria used for the rule.
                  returned: success
                  type: dict
                  contains:
                    protocol:
                      description: The protocol to inspect for.
                      returned: success
                      type: str
                      example: 'IP'
                    source:
                      description: The source address or range of addresses to inspect for.
                      returned: success
                      type: str
                      example: '203.0.113.98'
                    source_port:
                      description: The source port to inspect for.
                      returned: success
                      type: str
                      example: '42'
                    destination:
                      description: The destination address or range of addresses to inspect for.
                      returned: success
                      type: str
                      example: '198.51.100.0/24'
                    destination_port:
                      description: The destination port to inspect for.
                      returned: success
                      type: str
                      example: '6666:6667'
                    direction:
                      description: The direction of traffic flow to inspect.
                      returned: success
                      type: str
                      example: 'FORWARD'
                rule_options:
                  description: Additional Suricata RuleOptions settings for the rule.
                  returned: success
                  type: list
                  elements: dict
                  contains:
                    keyword:
                      description: The keyword for the setting.
                      returned: success
                      type: str
                      example: 'sid:1'
                    settings:
                      description: A list of values passed to the setting.
                      returned: When values are available
                      type: list
                      elements: str
            rules_string:
              description: A string describing the rules that the rule group is comprised of.
              returned: When the rule group is "rules string" based.
              type: str
            rules_source_list:
              description: A description of the criteria for a domain list rule group.
              returned: When the rule group is "domain list" based.
              type: dict
              contains:
                targets:
                  description: A list of domain names to be inspected for.
                  returned: success
                  type: list
                  elements: str
                  example: ['abc.example.com', '.example.net']
                target_types:
                  description: The protocols to be inspected by the rule group.
                  returned: success
                  type: list
                  elements: str
                  example: ['TLS_SNI', 'HTTP_HOST']
                generated_rules_type:
                  description: Whether the rule group allows or denies access to the domains in the list.
                  returned: success
                  type: str
                  example: 'ALLOWLIST'
            stateless_rules_and_custom_actions:
              description: A description of the criteria for a stateless rule group.
              returned: When the rule group is a stateless rule group.
              type: dict
              contains:
                stateless_rules:
                  description: A list of stateless rules for use in a stateless rule group.
                  type: list
                  elements: dict
                  contains:
                    rule_definition:
                      description: Describes the stateless 5-tuple inspection criteria and actions for the rule.
                      returned: success
                      type: dict
                      contains:
                        match_attributes:
                          description: Describes the stateless 5-tuple inspection criteria for the rule.
                          returned: success
                          type: dict
                          contains:
                            sources:
                              description: The source IP addresses and address ranges to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                address_definition:
                                  description: An IP address or a block of IP addresses in CIDR notation.
                                  returned: success
                                  type: str
                                  example: '192.0.2.3'
                            destinations:
                              description: The destination IP addresses and address ranges to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                address_definition:
                                  description: An IP address or a block of IP addresses in CIDR notation.
                                  returned: success
                                  type: str
                                  example: '192.0.2.3'
                            source_ports:
                              description: The source port ranges to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                from_port:
                                  description: The lower limit of the port range.
                                  returned: success
                                  type: int
                                to_port:
                                  description: The upper limit of the port range.
                                  returned: success
                                  type: int
                            destination_ports:
                              description: The destination port ranges to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                from_port:
                                  description: The lower limit of the port range.
                                  returned: success
                                  type: int
                                to_port:
                                  description: The upper limit of the port range.
                                  returned: success
                                  type: int
                            protocols:
                              description: The IANA protocol numbers of the protocols to inspect for.
                              returned: success
                              type: list
                              elements: int
                              example: [6]
                            tcp_flags:
                              description: The TCP flags and masks to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                flags:
                                  description: Used with masks to define the TCP flags that flows are inspected for.
                                  returned: success
                                  type: list
                                  elements: str
                                masks:
                                  description: The set of flags considered during inspection.
                                  returned: success
                                  type: list
                                  elements: str
                        actions:
                          description: The actions to take when a flow matches the rule.
                          returned: success
                          type: list
                          elements: str
                          example: ['aws:pass', 'CustomActionName']
                    priority:
                      description: Indicates the order in which to run this rule relative to all of the rules that are defined for a stateless rule group.
                      returned: success
                      type: int
                custom_actions:
                  description: A list of individual custom action definitions that are available for use in stateless rules.
                  type: list
                  elements: dict
                  contains:
                    action_name:
                      description: The name for the custom action.
                      returned: success
                      type: str
                    action_definition:
                      description: The custom action associated with the action name.
                      returned: success
                      type: dict
                      contains:
                        publish_metric_action:
                          description: The description of an action which publishes to CloudWatch.
                          returned: When the action publishes to CloudWatch.
                          type: dict
                          contains:
                            dimensions:
                              description: The value to use in an Amazon CloudWatch custom metric dimension.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                value:
                                  description: The value to use in the custom metric dimension.
                                  returned: success
                                  type: str
    rule_group_metadata:
      description: Details of the rules in the rule group
      type: dict
      returned: success
      contains:
        capacity:
          description: The maximum operating resources that this rule group can use.
          type: int
          returned: success
        consumed_capacity:
          description: The number of capacity units currently consumed by the rule group rules.
          type: int
          returned: success
        description:
          description: A description of the rule group.
          type: str
          returned: success
        number_of_associations:
          description: The number of firewall policies that use this rule group.
          type: int
          returned: success
        rule_group_arn:
          description: The ARN for the rule group
          type: int
          returned: success
          example: 'arn:aws:network-firewall:us-east-1:123456789012:stateful-rulegroup/ExampleGroup'
        rule_group_id:
          description: A unique identifier for the rule group.
          type: int
          returned: success
          example: '12345678-abcd-1234-abcd-123456789abc'
        rule_group_name:
          description: The name of the rule group.
          type: str
          returned: success
        rule_group_status:
          description: The current status of a rule group.
          type: str
          returned: success
          example: 'DELETING'
        tags:
          description: A dictionary representing the tags associated with the rule group.
          type: dict
          returned: success
        type:
          description: Whether the rule group is stateless or stateful.
          type: str
          returned: success
          example: 'STATEFUL'
"""

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.networkfirewall import NetworkFirewallRuleManager


def main():
    argument_spec = dict(
        name=dict(type="str", required=False),
        rule_type=dict(type="str", required=False, aliases=["type"], choices=["stateless", "stateful"]),
        arn=dict(type="str", required=False),
        scope=dict(type="str", required=False, choices=["managed", "account"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ["arn", "name"],
            ["arn", "rule_type"],
        ],
        required_together=[
            ["name", "rule_type"],
        ],
    )

    arn = module.params.get("arn")
    name = module.params.get("name")
    rule_type = module.params.get("rule_type")
    scope = module.params.get("scope")

    manager = NetworkFirewallRuleManager(module, name=name, rule_type=rule_type)

    results = dict(changed=False)

    if name or arn:
        rule = manager.get_rule_group(name=name, rule_type=rule_type, arn=arn)
        if rule:
            results["rule_groups"] = [rule]
        else:
            results["rule_groups"] = []
    else:
        rule_list = manager.list(scope=scope)
        results["rule_list"] = rule_list
        if scope != "managed":
            rules = [manager.get_rule_group(arn=r) for r in rule_list]
            results["rule_groups"] = rules

    module.exit_json(**results)


if __name__ == "__main__":
    main()



# (c) 2018 Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
author: Ansible Networking Team (@ansible-network)
name: eos
short_description: Use eAPI to run command on eos platform
description:
- This eos plugin provides low level abstraction api's for sending and receiving CLI
  commands with eos network devices.
version_added: 1.0.0
options:
  eos_use_sessions:
    type: bool
    default: yes
    description:
    - Specifies if sessions should be used on remote host or not
    env:
    - name: ANSIBLE_EOS_USE_SESSIONS
    vars:
    - name: ansible_eos_use_sessions
"""

import json

from ansible.errors import AnsibleConnectionFailure
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.connection import ConnectionError
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list
from ansible_collections.ansible.netcommon.plugins.plugin_utils.httpapi_base import HttpApiBase

from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import session_name


OPTIONS = {
    "format": ["text", "json"],
    "diff_match": ["line", "strict", "exact", "none"],
    "diff_replace": ["line", "block", "config"],
    "output": ["text", "json"],
}


class HttpApi(HttpApiBase):
    def __init__(self, *args, **kwargs):
        super(HttpApi, self).__init__(*args, **kwargs)
        self._device_info = None
        self._session_support = None

    def supports_sessions(self):
        if not self.get_option("eos_use_sessions"):
            self._session_support = False
        else:
            if self._session_support:
                return self._session_support

            try:
                response = self.send_request("show configuration sessions")
                self._session_support = "error" not in response
            except AnsibleConnectionFailure:
                self._session_support = False

        return self._session_support

    def send_request(self, data, **message_kwargs):
        data = to_list(data)
        become = self._become
        if become:
            self.connection.queue_message("vvvv", "firing event: on_become")
            data.insert(0, {"cmd": "enable", "input": self._become_pass})

        output = message_kwargs.get("output") or "text"
        version = message_kwargs.get("version") or "latest"
        request = request_builder(data, output, version)
        headers = {"Content-Type": "application/json-rpc"}

        _response, response_data = self.connection.send(
            "/command-api",
            request,
            headers=headers,
            method="POST",
        )

        try:
            response_data = json.loads(to_text(response_data.getvalue()))
        except ValueError:
            raise ConnectionError(
                "Response was not valid JSON, got {0}".format(
                    to_text(response_data.getvalue()),
                ),
            )

        results = handle_response(response_data)

        if become:
            results = results[1:]
        if len(results) == 1:
            results = results[0]

        return results

    def get_device_info(self):
        if self._device_info:
            return self._device_info

        device_info = {}

        device_info["network_os"] = "eos"
        reply = self.send_request("show version", output="json")
        data = json.loads(reply)

        device_info["network_os_version"] = data["version"]
        device_info["network_os_model"] = data["modelName"]

        reply = self.send_request("show hostname", output="json")
        data = json.loads(reply)

        device_info["network_os_hostname"] = data["hostname"]

        self._device_info = device_info
        return self._device_info

    def get_device_operations(self):
        return {
            "supports_diff_replace": True,
            "supports_commit": bool(self.supports_sessions()),
            "supports_rollback": False,
            "supports_defaults": False,
            "supports_onbox_diff": bool(self.supports_sessions()),
            "supports_commit_comment": False,
            "supports_multiline_delimiter": False,
            "supports_diff_match": True,
            "supports_diff_ignore_lines": True,
            "supports_generate_diff": not bool(self.supports_sessions()),
            "supports_replace": bool(self.supports_sessions()),
        }

    def get_capabilities(self):
        result = {}
        result["rpc"] = []
        result["device_info"] = self.get_device_info()
        result["device_operations"] = self.get_device_operations()
        result.update(OPTIONS)
        result["network_api"] = "eapi"

        return json.dumps(result)

    # Shims for resource module support
    def get(self, command, output=None):
        # This method is ONLY here to support resource modules. Therefore most
        # arguments are unsupported and not present.

        return self.send_request(data=command, output=output)

    def edit_config(self, candidate):
        # This method is ONLY here to support resource modules. Therefore most
        # arguments are unsupported and not present.

        session = None
        if self.supports_sessions():
            session = session_name()
            candidate = ["configure session %s" % session] + candidate
        else:
            candidate = ["configure"] + candidate
        candidate.append("commit")

        try:
            responses = self.send_request(candidate)
        except ConnectionError:
            if session:
                self.send_request(["configure session %s" % session, "abort"])
            raise

        return [resp for resp in to_list(responses) if resp != "{}"]


def handle_response(response):
    if "error" in response:
        error = response["error"]

        error_text = []
        for data in error.get("data", []):
            error_text.extend(data.get("errors", []))
        error_text = "\n".join(error_text) or error["message"]

        raise ConnectionError(error_text, code=error["code"])

    results = []

    for result in response["result"]:
        if "messages" in result:
            results.append(result["messages"][0])
        elif "output" in result:
            results.append(result["output"].strip())
        else:
            results.append(json.dumps(result))

    return results


def request_builder(commands, output, version, reqid=None):
    if version != "latest":
        version = int(version)
    params = dict(version=version, cmds=commands, format=output)
    return json.dumps(
        dict(jsonrpc="2.0", id=reqid, method="runCmds", params=params),
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Samita Bhattacharjee (@samiib)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: aci_switch_fabric_config
version_added: "2.13.0"
short_description: Manage Switch Fabric Policy Configuration of Leaf and Spine nodes (fabric:NodeConfig).
description:
- Manage Switch Fabric Policy Configuration of Leaf and Spine nodes (fabric:NodeConfig) on Cisco ACI fabrics.
- This module is only available for APIC version 6.0 and above.
options:
  node_type:
    description:
    - The type of Node.
    type: str
    aliases: [ type, switch_type ]
    choices: [ leaf, spine ]
  node:
    description:
    - The ID of the Node.
    - The value must be between 101 to 4000.
    type: int
    aliases: [ node_id ]
  policy_group:
    description:
    - The name of the Leaf/Spine Fabric Policy Group to associate with the node.
    - The Fabric Policy Group must exist for the settings to be applied.
    type: str
    aliases: [ fabric_policy_group, fabric_policy ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

seealso:
- module: cisco.aci.aci_fabric_switch_policy_group
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fabric:NodeConfig).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Samita Bhattacharjee (@samiib)
"""

EXAMPLES = r"""
- name: Add Switch Fabric Policy Configuration to a Leaf node
  cisco.aci.aci_switch_fabric_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    node: 101
    node_type: leaf
    policy_group: ansible_leaf_fabric_policy
    state: present
  delegate_to: localhost

- name: Query Switch Fabric Policy Configuration for a specific node
  cisco.aci.aci_switch_fabric_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    node: 101
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Switch Fabric Policy Configurations
  cisco.aci.aci_switch_fabric_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Remove a Switch Fabric Policy Configuration
  cisco.aci.aci_switch_fabric_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    node: 101
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: OK (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    aci_annotation_spec,
    switch_config_spec,
)
from ansible_collections.cisco.aci.plugins.module_utils.constants import (
    SWITCH_CONFIG_FORMAT_MAP,
)


def main():
    moClass = "fabricNodeConfig"
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(switch_config_spec(SWITCH_CONFIG_FORMAT_MAP[moClass]["type"]))
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["node"]],
            ["state", "present", ["node", "node_type", "policy_group"]],
        ],
    )

    node = module.params.get("node")
    node_type = module.params.get("node_type")
    policy_group = module.params.get("policy_group")
    state = module.params.get("state")

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class=moClass,
            target_filter=dict(node=node),
            aci_rn=SWITCH_CONFIG_FORMAT_MAP[moClass]["rn"].format(node),
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class=moClass,
            class_config=dict(
                node=node,
                assocGrp=SWITCH_CONFIG_FORMAT_MAP[moClass][node_type].format(policy_group),
            ),
        )

        aci.get_diff(aci_class=moClass)

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_devices_claim
short_description: Resource module for networks _devices _claim
description:
  - Manage operation create of the resource networks _devices _claim. - > Claim devices
    into a network. Note for recently claimed devices, it may take a few minutes for
    API requests against that device to succeed. This operation can be used up to
    ten times within a single five minute window.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  detailsByDevice:
    description: Optional details for claimed devices (currently only used for Catalyst
      devices).
    elements: dict
    suboptions:
      details:
        description: An array of details. Supported list of details includes "device
          mode", "username", "password", "enable password", "ap mapping type" and
          "ap network id". For onboarding into hybrid mode, the value of the device
          mode detail must be "monitored".
        elements: dict
        suboptions:
          name:
            description: Name of device detail.
            type: str
          value:
            description: Value of device detail.
            type: str
        type: list
      serial:
        description: The serial of the device these details relate to.
        type: str
    type: list
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  serials:
    description: A list of serials of devices to claim.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks claimNetworkDevices
    description: Complete reference of the claimNetworkDevices API.
    link: https://developer.cisco.com/meraki/api-v1/#!claim-network-devices
notes:
  - SDK Method used are
    networks.Networks.claim_network_devices,
  - Paths used are
    post /networks/{networkId}/devices/claim,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_devices_claim:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    addAtomically: true
    detailsByDevice:
      - details:
          - name: username
            value: milesmeraki
        serial: Q234-ABCD-5678
    networkId: string
    serials:
      - Q234-ABCD-0001
      - Q234-ABCD-0002
      - Q234-ABCD-0003
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "serials": [
        "string"
      ],
      "errors": [
        {
          "serial": "string",
          "errors": [
            "string"
          ]
        }
      ]
    }
"""



#!/usr/bin/python

# Copyright (c) 2022, Alexander Hussey <ahussey@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
"""
Ansible Module - community.general.keyring
"""

from __future__ import annotations

DOCUMENTATION = r"""
module: keyring
version_added: 5.2.0
author:
  - Alexander Hussey (@ahussey-redhat)
short_description: Set or delete a passphrase using the Operating System's native keyring
description: >-
  This module uses the L(keyring Python library, https://pypi.org/project/keyring/) to set or delete passphrases for a given
  service and username from the OS' native keyring.
requirements:
  - keyring (Python library)
  - gnome-keyring (application - required for headless Gnome keyring access)
  - dbus-run-session (application - required for headless Gnome keyring access)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  service:
    description: The name of the service.
    required: true
    type: str
  username:
    description: The user belonging to the service.
    required: true
    type: str
  user_password:
    description: The password to set.
    type: str
    aliases:
      - password
  keyring_password:
    description: Password to unlock keyring.
    required: true
    type: str
  state:
    description: Whether the password should exist.
    default: present
    type: str
    choices:
      - present
      - absent
"""

EXAMPLES = r"""
- name: Set a password for test/test1
  community.general.keyring:
    service: test
    username: test1
    user_password: "{{ user_password }}"
    keyring_password: "{{ keyring_password }}"

- name: Delete the password for test/test1
  community.general.keyring:
    service: test
    username: test1
    user_password: "{{ user_password }}"
    keyring_password: "{{ keyring_password }}"
    state: absent
"""

import traceback
from shlex import quote

from ansible.module_utils.basic import AnsibleModule, missing_required_lib

try:
    import keyring

    HAS_KEYRING = True
    KEYRING_IMP_ERR = None
except ImportError:
    HAS_KEYRING = False
    KEYRING_IMP_ERR = traceback.format_exc()


def del_passphrase(module):
    """
    Attempt to delete a passphrase in the keyring using the Python API and fallback to using a shell.
    """
    if module.check_mode:
        return None
    try:
        keyring.delete_password(module.params["service"], module.params["username"])
        return None
    except keyring.errors.KeyringLocked:
        delete_argument = (
            f'echo "{quote(module.params["keyring_password"])}" | gnome-keyring-daemon --unlock\n'
            f"keyring del {quote(module.params['service'])} {quote(module.params['username'])}\n"
        )
        dummy, dummy, stderr = module.run_command(
            "dbus-run-session -- /bin/bash",
            use_unsafe_shell=True,
            data=delete_argument,
            encoding=None,
        )

        if not stderr.decode("UTF-8"):
            return None
        return stderr.decode("UTF-8")


def set_passphrase(module):
    """
    Attempt to set passphrase in the keyring using the Python API and fallback to using a shell.
    """
    if module.check_mode:
        return None
    try:
        keyring.set_password(
            module.params["service"],
            module.params["username"],
            module.params["user_password"],
        )
        return None
    except keyring.errors.KeyringLocked:
        set_argument = (
            f'echo "{quote(module.params["keyring_password"])}" | gnome-keyring-daemon --unlock\n'
            f"keyring set {quote(module.params['service'])} {quote(module.params['username'])}\n{quote(module.params['user_password'])}\n"
        )
        dummy, dummy, stderr = module.run_command(
            "dbus-run-session -- /bin/bash",
            use_unsafe_shell=True,
            data=set_argument,
            encoding=None,
        )
        if not stderr.decode("UTF-8"):
            return None
        return stderr.decode("UTF-8")


def get_passphrase(module):
    """
    Attempt to retrieve passphrase from keyring using the Python API and fallback to using a shell.
    """
    try:
        passphrase = keyring.get_password(module.params["service"], module.params["username"])
        return passphrase
    except keyring.errors.KeyringLocked:
        pass
    except keyring.errors.InitError:
        pass
    except AttributeError:
        pass
    get_argument = (
        f'echo "{quote(module.params["keyring_password"])}" | gnome-keyring-daemon --unlock\n'
        f"keyring get {quote(module.params['service'])} {quote(module.params['username'])}\n"
    )
    dummy, stdout, dummy = module.run_command(
        "dbus-run-session -- /bin/bash",
        use_unsafe_shell=True,
        data=get_argument,
        encoding=None,
    )
    try:
        return stdout.decode("UTF-8").splitlines()[1]  # Only return the line containing the password
    except IndexError:
        return None


def run_module():
    """
    Attempts to retrieve a passphrase from a keyring.
    """
    result = dict(
        changed=False,
        msg="",
    )

    module_args = dict(
        service=dict(type="str", required=True),
        username=dict(type="str", required=True),
        keyring_password=dict(type="str", required=True, no_log=True),
        user_password=dict(type="str", no_log=True, aliases=["password"]),
        state=dict(type="str", default="present", choices=["absent", "present"]),
    )

    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    if not HAS_KEYRING:
        module.fail_json(msg=missing_required_lib("keyring"), exception=KEYRING_IMP_ERR)

    passphrase = get_passphrase(module)
    if module.params["state"] == "present":
        if passphrase is not None:
            if passphrase == module.params["user_password"]:
                result["msg"] = f"Passphrase already set for {module.params['service']}@{module.params['username']}"
            if passphrase != module.params["user_password"]:
                set_result = set_passphrase(module)
                if set_result is None:
                    result["changed"] = True
                    result["msg"] = (
                        f"Passphrase has been updated for {module.params['service']}@{module.params['username']}"
                    )
                if set_result is not None:
                    module.fail_json(msg=set_result)
        if passphrase is None:
            set_result = set_passphrase(module)
            if set_result is None:
                result["changed"] = True
                result["msg"] = (
                    f"Passphrase has been updated for {module.params['service']}@{module.params['username']}"
                )
            if set_result is not None:
                module.fail_json(msg=set_result)

    if module.params["state"] == "absent":
        if not passphrase:
            result["result"] = f"Passphrase already absent for {module.params['service']}@{module.params['username']}"
        if passphrase:
            del_result = del_passphrase(module)
            if del_result is None:
                result["changed"] = True
                result["msg"] = (
                    f"Passphrase has been removed for {module.params['service']}@{module.params['username']}"
                )
            if del_result is not None:
                module.fail_json(msg=del_result)

    module.exit_json(**result)


def main():
    """
    main module loop
    """
    run_module()


if __name__ == "__main__":
    main()



# Copyright (c) 2018 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
    name: azure_kql
    version_added: "3.7.0"
    short_description: Azure Resource Manager inventory plugin using Graph QL
    extends_documentation_fragment:
      - azure.azcollection.azure
      - azure.azcollection.azure_kql
      - constructed
      - inventory_cache
    description:
        - Query VM details from Azure Resource Manager using Graph QL
        - See https://learn.microsoft.com/en-us/azure/virtual-machines/resource-graph-samples?tabs=azure-cli
          for how to craft your own query.  The one requirement is that you need to provide inventory_hostname.
        - Requires a YAML configuration file whose name ends with 'azure_kql.(yml|yaml)'
        - Be aware that currently Azure Resource Graph may not be consistent with the actual state of your
          resources.  It can take up to 30 minutes for updates to propagate.  This applies both for resources
          to appear and to dissapear.
'''

EXAMPLES = '''
plugin: azure.azcollection.azure_kql

graph_query: |-
  Resources
      | where type =~ 'microsoft.compute/virtualmachines'
      | project vmId = tolower(tostring(id)),
                inventory_hostname = name,
                tags,
                location,
                resourceGroup,
                osType = tostring(properties.storageProfile.osDisk.osType),
                powerState = tostring(properties.extended.instanceView.powerState.displayStatus),
                hostName = properties.osProfile.computerName,
                subscription_id = subscriptionId
      | join kind=inner (ResourceContainers
          | where type=='microsoft.resources/subscriptions'
          | extend subscription_name = name,
                   subscription_id = subscriptionId,
                   state = properties.state
          | where state == 'Enabled'
          | project subscription_name,
                    subscription_id)
            on subscription_id
      | project-away subscription_id1
      | join (Resources
          | where type =~ 'microsoft.network/networkinterfaces'
          | mv-expand ipconfig=properties.ipConfigurations
          | project vmId = tolower(tostring(properties.virtualMachine.id)),
                    privateIp = ipconfig.properties.privateIPAddress,
                    publicIpId = tostring(ipconfig.properties.publicIPAddress.id)
          | join kind=leftouter (Resources
              | where type =~ 'microsoft.network/publicipaddresses'
              | project publicIpId = id, publicIp = properties.ipAddress
          ) on publicIpId
          | project-away publicIpId, publicIpId1
          | summarize privateIps = make_list(privateIp), publicIps = make_list(publicIp) by vmId
      ) on vmId
      | project-away vmId1
      | sort by inventory_hostname asc

# adds variables to each host found by this inventory plugin, whose values are the result of the associated expression
compose:
    ansible_host: "(publicIps + privateIps) | first"
    ansible_winrm_kerberos_hostname_override: "inventory_name + '.domain.tld'"
    ansible_winrm_transport: "'ntlm' if ('AAP_managed' in tags and ('DMZ' in (tags.AAP_Managed|list) or 'Local' in (tags.AAP_Managed|list))) else 'kerberos'"

groups:
    AAP_Managed: "'AAP_Managed' in (tags|list)"
    ubuntu18: "'AAP_Managed' in (tags|list) and 'ubuntu18' in tags.Ansible_OS"
    ubuntu20: "'AAP_Managed' in (tags|list) and 'ubuntu20' in tags.Ansible_OS"
    ubuntu22: "'AAP_Managed' in (tags|list) and 'ubuntu22' in tags.Ansible_OS"
    rhel7: "'AAP_Managed' in (tags|list) and 'rhel7' in tags.Ansible_OS"
    rhel8: "'AAP_Managed' in (tags|list) and 'rhel8' in tags.Ansible_OS"
    rhel9: "'AAP_Managed' in (tags|list) and 'rhel9' in tags.Ansible_OS"
    windows2012: "'AAP_Managed' in (tags|list) and 'windows2012' in tags.Ansible_OS"
    windows2016: "'AAP_Managed' in (tags|list) and 'windows2016' in tags.Ansible_OS"
    windows2019: "'AAP_Managed' in (tags|list) and 'windows2019' in tags.Ansible_OS"
    windows2022: "'AAP_Managed' in (tags|list) and 'windows2022' in tags.Ansible_OS"
    Asia: "'AAP_managed' in (tags|list) and 'Asia' in tags.AAP_Managed"
    North_America: "'AAP_managed' in (tags|list) and 'Asia' not in tags.AAP_Managed"

# change how inventory_hostname is generated. Each item is a jinja2 expression similar to hostvar_expressions.
hostnames:
    - "tags.vm_name if 'vm_name' in tags"
    - default_inventory_hostname + ".domain.tld" # Transfer to fqdn if you use shortnames for VMs
    - default  # special var that uses the default hashed name

keyed_groups:
    - prefix: ""
      separator: ""
      key: osType
    - prefix: ""
      separator: ""
      key: location
    - prefix: ""
      separator: ""
      key: powerState
'''

import re
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable, Cacheable
from ansible.errors import AnsibleError
from ansible.module_utils.parsing.convert_bool import boolean
from ansible.module_utils._text import to_native, to_text
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMAuth
from os import environ

try:
    import pandas as pd
    import azure.mgmt.resourcegraph as arg
except ImportError:
    pd = object
    arg = object
    pass


class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):

    NAME = 'azure.azcollection.azure_kql'

    def __init__(self):
        super(InventoryModule, self).__init__()

        self.azure_auth = None

    def verify_file(self, path):
        """ Verify inventory file """
        if super(InventoryModule, self).verify_file(path):
            if re.match(r'.{0,}azure_kql\.y(a)?ml$', path):
                return True
        raise AnsibleError("azure_kql inventory filename must end with 'azure_kql.yml' or 'azure_kql.yaml'")

    def parse(self, inventory, loader, path, cache=True):
        """ parses the inventory file """

        super(InventoryModule, self).parse(inventory, loader, path)

        self._read_config_data(path)

        # Load results from Cache if requested
        cache_key = self.get_cache_key(path)

        # cache may be True or False at this point to indicate if the inventory is being refreshed
        # get the user's cache option too to see if we should save the cache if it is changing
        user_cache_setting = self.get_option('cache')

        # read if the user has caching enabled and the cache isn't being refreshed
        attempt_to_read_cache = user_cache_setting and cache
        # update if the user has caching enabled and the cache is being refreshed;
        # update this value to True if the cache has expired below
        cache_needs_update = user_cache_setting and not cache

        # attempt to read the cache if inventory isn't being refreshed and the user has caching enabled
        if attempt_to_read_cache:
            try:
                results = self._cache[cache_key]
            except KeyError:
                # This occurs if the cache_key is not in the cache or if the cache_key
                # expired, so the cache needs to be updated
                cache_needs_update = True
        if not attempt_to_read_cache or cache_needs_update:
            # parse the provided inventory source
            try:
                self._credential_setup()
                results = self._get_hosts()
            except Exception:
                raise
        if cache_needs_update:
            self._cache[cache_key] = results

        self._populate(results)

    def _populate(self, results):
        """ Populate inventory """
        constructable_config_strict = boolean(self.get_option('fail_on_template_errors'))
        constructable_config_compose = self.get_option('compose')
        constructable_config_groups = self.get_option('groups')
        constructable_config_keyed_groups = self.get_option('keyed_groups')
        constructable_hostnames = self.get_option('hostnames')

        for h in results:
            hostvars = h.get("hostvars")
            inventory_hostname = self._get_hostname(h,
                                                    hostnames=constructable_hostnames,
                                                    strict=constructable_config_strict)
            self.inventory.add_host(inventory_hostname)

            for k, v in hostvars.items():
                self.inventory.set_variable(inventory_hostname, k, v)

            # constructable delegation
            self._set_composite_vars(constructable_config_compose,
                                     hostvars,
                                     inventory_hostname,
                                     strict=constructable_config_strict)
            self._add_host_to_composed_groups(constructable_config_groups,
                                              hostvars,
                                              inventory_hostname,
                                              strict=constructable_config_strict)
            self._add_host_to_keyed_groups(constructable_config_keyed_groups,
                                           hostvars,
                                           inventory_hostname,
                                           strict=constructable_config_strict)

    def _get_hostname(self, host, hostnames=None, strict=False):
        hostname = None
        errors = []

        for preference in hostnames:
            if preference == 'default':
                return host.get("default_inventory_hostname")
            try:
                hostname = self._compose(preference, host.get("hostvars"))
            except Exception as e:  # pylint: disable=broad-except
                if strict:
                    raise AnsibleError("Could not compose %s as hostnames - %s" % (preference, to_native(e)))
                else:
                    errors.append(
                        (preference, str(e))
                    )
            if hostname:
                return to_text(hostname)

        raise AnsibleError(
            'Could not template any hostname for host, errors for each preference: %s' % (
                ', '.join(['%s: %s' % (pref, err) for pref, err in errors])
            )
        )

    def _credential_setup(self):
        auth_source = environ.get('ANSIBLE_AZURE_AUTH_SOURCE', None) or self.get_option('auth_source')
        auth_options = dict(
            auth_source=auth_source,
            profile=self.get_option('profile'),
            subscription_id=self.get_option('subscription_id'),
            client_id=self.get_option('client_id'),
            secret=self.get_option('secret'),
            tenant=self.get_option('tenant'),
            ad_user=self.get_option('ad_user'),
            password=self.get_option('password'),
            cloud_environment=self.get_option('cloud_environment'),
            cert_validation_mode=self.get_option('cert_validation_mode'),
            api_profile=self.get_option('api_profile'),
            track1_cred=True,
            adfs_authority_url=self.get_option('adfs_authority_url')
        )

        if self.templar.is_template(auth_options["tenant"]):
            auth_options["tenant"] = self.templar.template(variable=auth_options["tenant"], disable_lookups=False)

        if self.templar.is_template(auth_options["client_id"]):
            auth_options["client_id"] = self.templar.template(variable=auth_options["client_id"], disable_lookups=False)

        if self.templar.is_template(auth_options["secret"]):
            auth_options["secret"] = self.templar.template(variable=auth_options["secret"], disable_lookups=False)

        if self.templar.is_template(auth_options["subscription_id"]):
            auth_options["subscription_id"] = self.templar.template(variable=auth_options["subscription_id"], disable_lookups=False)

        self.azure_auth = AzureRMAuth(**auth_options)

    def execute_kql(self, query, resource_name='VMs'):
        """ Execute KQL query """

        argClient = arg.ResourceGraphClient(self.azure_auth.azure_credential_track2)
        skpToken = 'hasData'
        output = []

        while skpToken is not None:
            if skpToken == 'hasData':
                argQueryOptions = arg.models.QueryRequestOptions(result_format="objectArray")
            else:
                argQueryOptions = arg.models.QueryRequestOptions(result_format="objectArray", skip_token=skpToken)
            argQuery = arg.models.QueryRequest(query=query, options=argQueryOptions)
            argResults = argClient.resources(argQuery)
            output.extend(argResults.data)
            skpToken = argResults.skip_token

        df_output = pd.DataFrame(output)
        return df_output

    def _get_hosts(self):
        """ Get all hosts via graph_query """

        df_vms = self.execute_kql(query=self.get_option('graph_query'))
        results = []

        for index, row in df_vms.iterrows():
            # Convert panda object to dict
            row = row.to_dict()
            # If no tags are present use an empty dict
            tags = row.pop('tags') or {}
            # Update row with updated tags
            row.update({'tags': tags})
            results.append(dict(default_inventory_hostname=row.get('inventory_hostname'),
                                hostvars=row))

        return results



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: eos_config
author: Peter Sprygada (@privateip)
short_description: Manage Arista EOS configuration sections
description:
- Arista EOS configurations use a simple block indent file syntax for segmenting configuration
  into sections.  This module provides an implementation for working with EOS configuration
  sections in a deterministic way.  This module works with either CLI or eAPI transports.
version_added: 1.0.0
notes:
- Tested against Arista EOS 4.24.6F
- Abbreviated commands are NOT idempotent, see
  L(Network FAQ,../network/user_guide/faq.html#why-do-the-config-modules-always-return-changed-true-with-abbreviated-commands).
- To ensure idempotency and correct diff the configuration lines in the relevant module options should be similar to how they
  appear if present in the running configuration on device including the indentation.
options:
  lines:
    description:
    - The ordered set of commands that should be configured in the section. The commands
      must be the exact same commands as found in the device running-config as found in the
      device running-config to ensure idempotency and correct diff. Be sure
      to note the configuration command syntax as some commands are automatically
      modified by the device config parser.
    aliases:
    - commands
    type: list
    elements: str
  parents:
    description:
    - The ordered set of parents that uniquely identify the section or hierarchy the
      commands should be checked against.  If the parents argument is omitted, the
      commands are checked against the set of top level or global commands.
    type: list
    elements: str
  src:
    description:
    - Specifies the source path to the file that contains the configuration or configuration
      template to load.  The path to the source file can either be the full path on
      the Ansible control host or a relative path from the playbook or role root directory. This
      argument is mutually exclusive with I(lines), I(parents), and I(content). The configuration lines in the
      source file should be similar to how it will appear if present in the running-configuration
      (live switch config) of the device including the indentation to ensure idempotency
      and correct diff. Arista EOS device config has 3 spaces indentation.
    - "NOTE: The I(src) parameter will no longer process Jinja2 templates starting in January 2028.
      To use templated configurations, render the template using C(ansible.builtin.template) lookup
      and pass the result to the I(content) parameter instead."
    type: path
  content:
    description:
    - Configuration content to apply to the device. This should be the rendered configuration
      text, not a file path.
    - This is the recommended way to provide templated configurations. Use C(ansible.builtin.template)
      lookup to render your Jinja2 template and pass the output to this parameter.
    - This argument is mutually exclusive with I(src), I(lines), and I(parents).
    - 'Example: C(content: "{{ lookup(''ansible.builtin.template'', ''config.j2'') }}")'
    type: str
  before:
    description:
    - The ordered set of commands to push on to the command stack if a change needs
      to be made.  This allows the playbook designer the opportunity to perform configuration
      commands prior to pushing any changes without affecting how the set of commands
      are matched against the system.
    type: list
    elements: str
  after:
    description:
    - The ordered set of commands to append to the end of the command stack if a change
      needs to be made.  Just like with I(before) this allows the playbook designer
      to append a set of commands to be executed after the command set.
    type: list
    elements: str
  match:
    description:
    - Instructs the module on the way to perform the matching of the set of commands
      against the current device config.  If match is set to I(line), commands are
      matched line by line.  If match is set to I(strict), command lines are matched
      with respect to position.  If match is set to I(exact), command lines must be
      an equal match.  Finally, if match is set to I(none), the module will not attempt
      to compare the source configuration with the running configuration on the remote
      device.
    type: str
    default: line
    choices:
    - line
    - strict
    - exact
    - none
  replace:
    description:
    - Instructs the module on the way to perform the configuration on the device.  If
      the replace argument is set to I(line) then the modified lines are pushed to
      the device in configuration mode.  If the replace argument is set to I(block)
      then the entire command block is pushed to the device in configuration mode
      if any line is not correct.
    type: str
    default: line
    choices:
    - line
    - block
    - config
  backup:
    description:
    - This argument will cause the module to create a full backup of the current C(running-config)
      from the remote device before any changes are made. If the C(backup_options)
      value is not given, the backup file is written to the C(backup) folder in the
      playbook root directory or role root directory, if playbook is part of an ansible
      role. If the directory does not exist, it is created.
    type: bool
    default: false
  running_config:
    description:
    - The module, by default, will connect to the remote device and retrieve the current
      running-config to use as a base for comparing against the contents of source.  There
      are times when it is not desirable to have the task get the current running-config
      for every task in a playbook.  The I(running_config) argument allows the implementer
      to pass in the configuration to use as the base config for this module.
      The configuration lines for this option should be similar to how it will appear if present
      in the running-configuration of the device including the indentation to ensure idempotency
      and correct diff.
    type: str
    aliases:
    - config
  defaults:
    description:
    - The I(defaults) argument will influence how the running-config is collected
      from the device.  When the value is set to true, the command used to collect
      the running-config is append with the all keyword.  When the value is set to
      false, the command is issued without the all keyword
    type: bool
    default: false
  save_when:
    description:
    - When changes are made to the device running-configuration, the changes are not
      copied to non-volatile storage by default.  Using this argument will change
      that before.  If the argument is set to I(always), then the running-config will
      always be copied to the startup-config and the I(modified) flag will always
      be set to True.  If the argument is set to I(modified), then the running-config
      will only be copied to the startup-config if it has changed since the last save
      to startup-config.  If the argument is set to I(never), the running-config will
      never be copied to the startup-config. If the argument is set to I(changed),
      then the running-config will only be copied to the startup-config if the task
      has made a change. I(changed) was added in Ansible 2.5.
    default: never
    type: str
    choices:
    - always
    - never
    - modified
    - changed
  diff_against:
    description:
    - When using the C(ansible-playbook --diff) command line argument the module can
      generate diffs against different sources.
    - When this option is configure as I(startup), the module will return the diff
      of the running-config against the startup-config.
    - When this option is configured as I(intended), the module will return the diff
      of the running-config against the configuration provided in the C(intended_config)
      argument.
    - When this option is configured as I(running), the module will return the before
      and after diff of the running-config with respect to any changes made to the
      device configuration.
    - When this option is configured as C(session), the diff returned will be based
      on the configuration session.
    - When this option is configured as C(validate_config), the module will return before
      with the running-config before applying the intended config and after with the session
      config after applying the intended config to the session.
    default: session
    type: str
    choices:
    - startup
    - running
    - intended
    - session
    - validate_config
  diff_ignore_lines:
    description:
    - Use this argument to specify one or more lines that should be ignored during
      the diff.  This is used for lines in the configuration that are automatically
      updated by the system.  This argument takes a list of regular expressions or
      exact line matches.
    type: list
    elements: str
  intended_config:
    description:
    - The C(intended_config) provides the master configuration that the node should
      conform to and is used to check the final running-config against.   This argument
      will not modify any settings on the remote device and is strictly used to check
      the compliance of the current device's configuration against.  When specifying
      this argument, the task should also modify the C(diff_against) value and set
      it to I(intended). The configuration lines for this value should be similar to how it
      will appear if present in the running-configuration of the device including the indentation
      to ensure correct diff.
    type: str
  backup_options:
    description:
    - This is a dict object containing configurable options related to backup file
      path. The value of this option is read only when C(backup) is set to I(true),
      if C(backup) is set to I(no) this option will be silently ignored.
    suboptions:
      filename:
        description:
        - The filename to be used to store the backup configuration. If the filename
          is not given it will be generated based on the hostname, current time and
          date in format defined by <hostname>_config.<current-date>@<current-time>
        type: str
      dir_path:
        description:
        - This option provides the path ending with directory name in which the backup
          configuration file will be stored. If the directory does not exist it will
          be first created and the filename is either the value of C(filename) or
          default filename as described in C(filename) options description. If the
          path value is not given in that case a I(backup) directory will be created
          in the current working directory and backup configuration will be copied
          in C(filename) within I(backup) directory.
        type: path
    type: dict
"""
# noqa: E501

EXAMPLES = """
- name: configure top level settings
  arista.eos.eos_config:
    lines: hostname {{ inventory_hostname }}

- name: load an acl into the device
  arista.eos.eos_config:
    lines:
      - 10 permit ip host 192.0.2.1 any log
      - 20 permit ip host 192.0.2.2 any log
      - 30 permit ip host 192.0.2.3 any log
      - 40 permit ip host 192.0.2.4 any log
    parents: ip access-list test
    before: no ip access-list test
    replace: block

- name: load configuration from file
  arista.eos.eos_config:
    src: eos.cfg

- name: Render a Jinja2 template onto an Arista switch (DEPRECATED - use content parameter)
  arista.eos.eos_config:
    backup: true
    src: eos_template.j2

- name: Apply templated configuration using content parameter (RECOMMENDED)
  arista.eos.eos_config:
    content: "{{ lookup('ansible.builtin.template', 'eos_template.j2') }}"
    backup: true

- name: Apply templated configuration with backup options (RECOMMENDED)
  arista.eos.eos_config:
    content: "{{ lookup('ansible.builtin.template', 'eos_template.j2') }}"
    backup: true
    backup_options:
      filename: backup.cfg
      dir_path: /home/user

- name: Load configuration from pre-rendered template
  arista.eos.eos_config:
    content: |
      interface Ethernet1
         description Uplink to Core
         ip address 10.1.1.1/24
         no shutdown

- name: diff the running config against a master config
  arista.eos.eos_config:
    diff_against: intended
    intended_config: "{{ lookup('file', 'master.cfg') }}"

- name: for idempotency, use full-form commands
  arista.eos.eos_config:
    lines:
      - shutdown
    parents: interface Ethernet1

- name: configurable backup path
  arista.eos.eos_config:
    src: eos_template.j2
    backup: true
    backup_options:
      filename: backup.cfg
      dir_path: /home/user
"""

RETURN = """
commands:
  description: The set of commands that will be pushed to the remote device
  returned: always
  type: list
  sample: ['hostname switch01', 'interface Ethernet1', 'no shutdown']
updates:
  description: The set of commands that will be pushed to the remote device
  returned: always
  type: list
  sample: ['hostname switch01', 'interface Ethernet1', 'no shutdown']
backup_path:
  description: The full path to the backup file
  returned: when backup is true
  type: str
  sample: /playbooks/ansible/backup/eos_config.2016-07-16@22:28:34
filename:
  description: The name of the backup file
  returned: when backup is true and filename is not specified in backup options
  type: str
  sample: eos_config.2016-07-16@22:28:34
shortname:
  description: The full path to the backup file excluding the timestamp
  returned: when backup is true and filename is not specified in backup options
  type: str
  sample: /playbooks/ansible/backup/eos_config
date:
  description: The date extracted from the backup file name
  returned: when backup is true
  type: str
  sample: "2016-07-16"
time:
  description: The time extracted from the backup file name
  returned: when backup is true
  type: str
  sample: "22:28:34"
"""
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.connection import ConnectionError
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    NetworkConfig,
    dumps,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import (
    get_config,
    get_connection,
    get_session_config,
    load_config,
    run_commands,
)


def get_candidate(module):
    candidate = ""
    if module.params["src"]:
        candidate = module.params["src"]
    elif module.params["content"]:
        candidate = module.params["content"]
    elif module.params["lines"]:
        candidate_obj = NetworkConfig(indent=3)
        parents = module.params["parents"] or list()
        candidate_obj.add(module.params["lines"], parents=parents)
        candidate = dumps(candidate_obj, "raw")
    return candidate


def get_running_config(module, config=None, flags=None):
    contents = module.params["running_config"]
    if not contents:
        if config:
            contents = config
        else:
            contents = get_config(module, flags=flags)
    return contents


def save_config(module, result):
    result["changed"] = True
    if not module.check_mode:
        cmd = {
            "command": "copy running-config startup-config",
            "output": "text",
        }
        run_commands(module, [cmd])
    else:
        module.warn(
            "Skipping command `copy running-config startup-config` "
            "due to check_mode.  Configuration not copied to "
            "non-volatile storage",
        )


def main():
    """main entry point for module execution"""
    backup_spec = dict(filename=dict(), dir_path=dict(type="path"))
    argument_spec = dict(
        src=dict(type="path"),
        content=dict(type="str"),
        lines=dict(aliases=["commands"], type="list", elements="str"),
        parents=dict(type="list", elements="str"),
        before=dict(type="list", elements="str"),
        after=dict(type="list", elements="str"),
        match=dict(
            default="line",
            choices=["line", "strict", "exact", "none"],
        ),
        replace=dict(default="line", choices=["line", "block", "config"]),
        defaults=dict(type="bool", default=False),
        backup=dict(type="bool", default=False),
        backup_options=dict(type="dict", options=backup_spec),
        save_when=dict(
            choices=["always", "never", "modified", "changed"],
            default="never",
        ),
        diff_against=dict(
            choices=[
                "startup",
                "session",
                "intended",
                "running",
                "validate_config",
            ],
            default="session",
        ),
        diff_ignore_lines=dict(type="list", elements="str"),
        running_config=dict(aliases=["config"]),
        intended_config=dict(),
    )

    mutually_exclusive = [("lines", "src", "content"), ("parents", "src", "content")]

    required_if = [
        ("match", "strict", ["lines", "src", "content"], True),
        ("match", "exact", ["lines", "src", "content"], True),
        ("replace", "block", ["lines", "src", "content"], True),
        ("replace", "config", ["src"]),
        ("diff_against", "intended", ["intended_config"]),
        ("diff_against", "validate_config", ["intended_config"]),
    ]

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        required_if=required_if,
        supports_check_mode=True,
    )

    warnings = list()

    result = {"changed": False}
    if warnings:
        result["warnings"] = warnings

    diff_ignore_lines = module.params["diff_ignore_lines"]
    config = None
    contents = None
    flags = ["all"] if module.params["defaults"] else []
    connection = get_connection(module)
    # Refuse to diff_against: session if sessions are disabled
    if (
        module.params["diff_against"] in ["session", "validate_config"]
        and not connection.supports_sessions
    ):
        module.fail_json(
            msg="Cannot diff against sessions when sessions are disabled. Please change diff_against to another value",
        )

    if module.params["backup"] or (module._diff and module.params["diff_against"] == "running"):
        contents = get_config(module, flags=flags)
        config = NetworkConfig(indent=1, contents=contents)
        if module.params["backup"]:
            result["__backup__"] = contents

    if any((module.params["src"], module.params["lines"], module.params["content"])):
        match = module.params["match"]
        replace = module.params["replace"]
        path = module.params["parents"]

        candidate = get_candidate(module)
        running = get_running_config(module, contents, flags=flags)

        try:
            response = connection.get_diff(
                candidate=candidate,
                running=running,
                diff_match=match,
                diff_ignore_lines=diff_ignore_lines,
                path=path,
                diff_replace=replace,
            )
        except ConnectionError as exc:
            module.fail_json(msg=to_text(exc, errors="surrogate_then_replace"))

        config_diff = response["config_diff"]
        if config_diff:
            commands = config_diff.split("\n")
            if module.params["before"]:
                commands[:0] = module.params["before"]

            if module.params["after"]:
                commands.extend(module.params["after"])

            result["commands"] = commands
            result["updates"] = commands

            replace = module.params["replace"] == "config"
            commit = not module.check_mode

            response = load_config(
                module,
                commands,
                replace=replace,
                commit=commit,
            )

            result["changed"] = True

            if module.params["diff_against"] == "session":
                if "diff" in response:
                    result["diff"] = {"prepared": response["diff"]}
                else:
                    result["changed"] = False

            if "session" in response:
                result["session"] = response["session"]

    running_config = module.params["running_config"]
    startup_config = None
    if module.params["save_when"] == "always":
        save_config(module, result)
    elif module.params["save_when"] == "modified":
        output = run_commands(
            module,
            [
                {"command": "show running-config", "output": "text"},
                {"command": "show startup-config", "output": "text"},
            ],
        )

        running_config = NetworkConfig(
            indent=3,
            contents=output[0],
            ignore_lines=diff_ignore_lines,
        )
        startup_config = NetworkConfig(
            indent=3,
            contents=output[1],
            ignore_lines=diff_ignore_lines,
        )

        if running_config.sha1 != startup_config.sha1:
            save_config(module, result)

    elif module.params["save_when"] == "changed" and result["changed"]:
        save_config(module, result)
    if module._diff:
        if not running_config:
            output = run_commands(
                module,
                {"command": "show running-config", "output": "text"},
            )
            contents = output[0]
        else:
            contents = running_config

        # recreate the object in order to process diff_ignore_lines
        running_config = NetworkConfig(
            indent=3,
            contents=contents,
            ignore_lines=diff_ignore_lines,
        )

        if module.params["diff_against"] == "running":
            if module.check_mode:
                module.warn(
                    "unable to perform diff against running-config due to check mode",
                )
                contents = None
            else:
                contents = config.config_text

        elif module.params["diff_against"] == "startup":
            if not startup_config:
                output = run_commands(
                    module,
                    {"command": "show startup-config", "output": "text"},
                )
                contents = output[0]
            else:
                contents = startup_config.config_text

        elif module.params["diff_against"] in ["intended", "validate_config"]:
            contents = module.params["intended_config"]

        if contents is not None:
            base_config = NetworkConfig(
                indent=3,
                contents=contents,
                ignore_lines=diff_ignore_lines,
            )

            if running_config.sha1 != base_config.sha1:
                before = ""
                after = ""
                if module.params["diff_against"] == "intended":
                    before = running_config
                    after = base_config
                elif module.params["diff_against"] in ("startup", "running"):
                    before = base_config
                    after = running_config
                elif module.params["diff_against"] == "validate_config":
                    before = running = get_running_config(
                        module,
                        None,
                        flags=flags,
                    )
                    replace = module.params["replace"] == "config"
                    after = get_session_config(
                        module,
                        contents.split("\n"),
                        replace=replace,
                        commit=False,
                    )

                result.update(
                    {
                        "changed": False,
                        "diff": {"before": str(before), "after": str(after)},
                    },
                )

    if result.get("changed") and any(
        (module.params["src"], module.params["lines"], module.params["content"]),
    ):
        msg = (
            "To ensure idempotency and correct diff the input configuration lines should be"
            " similar to how they appear if present in"
            " the running configuration on device"
        )
        if module.params["src"] or module.params["content"]:
            msg += " including the indentation"
        if "warnings" in result:
            result["warnings"].append(msg)
        else:
            result["warnings"] = msg

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (C) 2013, Peter Sprygada <sprygada@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ejabberd_user
author: "Peter Sprygada (@privateip)"
short_description: Manages users for ejabberd servers
requirements:
  - ejabberd with mod_admin_extra
description:
  - This module provides user management for ejabberd servers.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  username:
    type: str
    description:
      - The name of the user to manage.
    required: true
  host:
    type: str
    description:
      - The ejabberd host associated with this username.
    required: true
  password:
    type: str
    description:
      - The password to assign to the username.
  state:
    type: str
    description:
      - Describe the desired state of the user to be managed.
    default: 'present'
    choices: ['present', 'absent']
notes:
  - Password parameter is required for O(state=present) only.
  - Passwords must be stored in clear text for this release.
  - The ejabberd configuration file must include mod_admin_extra as a module.
"""
EXAMPLES = r"""
# Example playbook entries using the ejabberd_user module to manage users state.

- name: Create a user if it does not exist
  community.general.ejabberd_user:
    username: test
    host: server
    password: password

- name: Delete a user if it exists
  community.general.ejabberd_user:
    username: test
    host: server
    state: absent
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._cmd_runner import CmdRunner, cmd_runner_fmt


class EjabberdUser:
    """This object represents a user resource for an ejabberd server.   The
    object manages user creation and deletion using ejabberdctl.  The following
    commands are currently supported:
        * ejabberdctl register
        * ejabberdctl unregister
    """

    def __init__(self, module):
        self.module = module
        self.state = module.params.get("state")
        self.host = module.params.get("host")
        self.user = module.params.get("username")
        self.pwd = module.params.get("password")
        self.runner = CmdRunner(
            module,
            command="ejabberdctl",
            arg_formats=dict(
                cmd=cmd_runner_fmt.as_list(),
                host=cmd_runner_fmt.as_list(),
                user=cmd_runner_fmt.as_list(),
                pwd=cmd_runner_fmt.as_list(),
            ),
            check_rc=False,
        )

    @property
    def changed(self):
        """This method will check the current user and see if the password has
        changed.   It will return True if the user does not match the supplied
        credentials and False if it does not
        """
        return self.run_command("check_password", "user host pwd", (lambda rc, out, err: bool(rc)))

    @property
    def exists(self):
        """This method will check to see if the supplied username exists for
        host specified.  If the user exists True is returned, otherwise False
        is returned
        """
        return self.run_command("check_account", "user host", (lambda rc, out, err: not bool(rc)))

    def log(self, entry):
        """This method does nothing"""
        pass

    def run_command(self, cmd, options, process=None):
        """This method will run the any command specified and return the
        returns using the Ansible common module
        """

        def _proc(*a):
            return a

        if process is None:
            process = _proc

        with self.runner(f"cmd {options}", output_process=process) as ctx:
            res = ctx.run(cmd=cmd, host=self.host, user=self.user, pwd=self.pwd)
            self.log(f"command: {' '.join(ctx.run_info['cmd'])}")
        return res

    def update(self):
        """The update method will update the credentials for the user provided"""
        return self.run_command("change_password", "user host pwd")

    def create(self):
        """The create method will create a new user on the host with the
        password provided
        """
        return self.run_command("register", "user host pwd")

    def delete(self):
        """The delete method will delete the user from the host"""
        return self.run_command("unregister", "user host")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            host=dict(required=True, type="str"),
            username=dict(required=True, type="str"),
            password=dict(type="str", no_log=True),
            state=dict(default="present", choices=["present", "absent"]),
        ),
        required_if=[
            ("state", "present", ["password"]),
        ],
        supports_check_mode=True,
    )

    obj = EjabberdUser(module)

    rc = None
    result = dict(changed=False)

    if obj.state == "absent":
        if obj.exists:
            if module.check_mode:
                module.exit_json(changed=True)
            (rc, out, err) = obj.delete()
            if rc != 0:
                module.fail_json(msg=err, rc=rc)

    elif obj.state == "present":
        if not obj.exists:
            if module.check_mode:
                module.exit_json(changed=True)
            (rc, out, err) = obj.create()
        elif obj.changed:
            if module.check_mode:
                module.exit_json(changed=True)
            (rc, out, err) = obj.update()
        if rc is not None and rc != 0:
            module.fail_json(msg=err, rc=rc)

    if rc is None:
        result["changed"] = False
    else:
        result["changed"] = True

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Aaron Smith <ajsmith10381@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: config_rule
version_added: 1.0.0
short_description: Manage AWS Config rule resources
description:
  - Module manages AWS Config rules.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_config_rule).
    The usage did not change.
author:
  - "Aaron Smith (@slapula)"
options:
  name:
    description:
      - The name of the AWS Config resource.
    required: true
    type: str
  state:
    description:
      - Whether the Config rule should be present or absent.
    default: present
    choices: ['present', 'absent']
    type: str
  description:
    description:
      - The description that you provide for the AWS Config rule.
    type: str
  scope:
    description:
      - Defines which resources can trigger an evaluation for the rule.
    suboptions:
      compliance_types:
        description:
          - The resource types of only those AWS resources that you want to trigger an evaluation for the rule.
            You can only specify one type if you also specify a resource ID for I(compliance_id).
      compliance_id:
        description:
          - The ID of the only AWS resource that you want to trigger an evaluation for the rule. If you specify a resource ID,
            you must specify one resource type for I(compliance_types).
      tag_key:
        description:
          - The tag key that is applied to only those AWS resources that you want to trigger an evaluation for the rule.
      tag_value:
        description:
          - The tag value applied to only those AWS resources that you want to trigger an evaluation for the rule.
            If you specify a value for I(tag_value), you must also specify a value for I(tag_key).
    type: dict
  source:
    description:
      - Provides the rule owner (AWS or customer), the rule identifier, and the notifications that cause the function to
        evaluate your AWS resources.
    suboptions:
      owner:
        description:
          - The resource types of only those AWS resources that you want to trigger an evaluation for the rule.
            You can only specify one type if you also specify a resource ID for I(compliance_id).
      identifier:
        description:
          - The ID of the only AWS resource that you want to trigger an evaluation for the rule.
            If you specify a resource ID, you must specify one resource type for I(compliance_types).
      details:
        description:
          - Provides the source and type of the event that causes AWS Config to evaluate your AWS resources.
          - This parameter expects a list of dictionaries.  Each dictionary expects the following key/value pairs.
          - Key C(EventSource) The source of the event, such as an AWS service, that triggers AWS Config to evaluate your AWS resources.
          - Key C(MessageType) The type of notification that triggers AWS Config to run an evaluation for a rule.
          - Key C(MaximumExecutionFrequency) The frequency at which you want AWS Config to run evaluations for a custom rule with a periodic trigger.
    type: dict
    required: true
  input_parameters:
    description:
      - A string, in JSON format, that is passed to the AWS Config rule Lambda function.
    type: str
  execution_frequency:
    description:
      - The maximum frequency with which AWS Config runs evaluations for a rule.
    choices: ['One_Hour', 'Three_Hours', 'Six_Hours', 'Twelve_Hours', 'TwentyFour_Hours']
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create Config Rule for AWS Config
  community.aws.config_rule:
    name: test_config_rule
    state: present
    description: 'This AWS Config rule checks for public write access on S3 buckets'
    scope:
      compliance_types:
        - 'AWS::S3::Bucket'
    source:
      owner: AWS
      identifier: 'S3_BUCKET_PUBLIC_WRITE_PROHIBITED'
"""

RETURN = r"""#"""


try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def rule_exists(client, module, params):
    try:
        rule = client.describe_config_rules(
            ConfigRuleNames=[params["ConfigRuleName"]],
            aws_retry=True,
        )
        return rule["ConfigRules"][0]
    except is_boto3_error_code("NoSuchConfigRuleException"):
        return
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e)


def create_resource(client, module, params, result):
    try:
        client.put_config_rule(ConfigRule=params)
        result["changed"] = True
        return result
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't create AWS Config rule")


def update_resource(client, module, params, result):
    current_params = client.describe_config_rules(
        ConfigRuleNames=[params["ConfigRuleName"]],
        aws_retry=True,
    )

    del current_params["ConfigRules"][0]["ConfigRuleArn"]
    del current_params["ConfigRules"][0]["ConfigRuleId"]
    del current_params["ConfigRules"][0]["EvaluationModes"]

    if params != current_params["ConfigRules"][0]:
        try:
            client.put_config_rule(ConfigRule=params)
            result["changed"] = True
            result["rule"] = camel_dict_to_snake_dict(rule_exists(client, module, params))
            return result
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't create AWS Config rule")


def delete_resource(client, module, params, result):
    try:
        client.delete_config_rule(
            ConfigRuleName=params["ConfigRuleName"],
            aws_retry=True,
        )
        result["changed"] = True
        result["rule"] = {}
        return result
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't delete AWS Config rule")


def main():
    module = AnsibleAWSModule(
        argument_spec={
            "name": dict(type="str", required=True),
            "state": dict(type="str", choices=["present", "absent"], default="present"),
            "description": dict(type="str"),
            "scope": dict(type="dict"),
            "source": dict(type="dict", required=True),
            "input_parameters": dict(type="str"),
            "execution_frequency": dict(
                type="str",
                choices=[
                    "One_Hour",
                    "Three_Hours",
                    "Six_Hours",
                    "Twelve_Hours",
                    "TwentyFour_Hours",
                ],
            ),
        },
        supports_check_mode=False,
    )

    result = {"changed": False}

    name = module.params.get("name")
    state = module.params.get("state")

    params = {}
    if name:
        params["ConfigRuleName"] = name
    if module.params.get("description"):
        params["Description"] = module.params.get("description")
    if module.params.get("scope"):
        params["Scope"] = {}
        if module.params.get("scope").get("compliance_types"):
            params["Scope"].update(
                {
                    "ComplianceResourceTypes": module.params.get("scope").get("compliance_types"),
                }
            )
        if module.params.get("scope").get("tag_key"):
            params["Scope"].update(
                {
                    "TagKey": module.params.get("scope").get("tag_key"),
                }
            )
        if module.params.get("scope").get("tag_value"):
            params["Scope"].update(
                {
                    "TagValue": module.params.get("scope").get("tag_value"),
                }
            )
        if module.params.get("scope").get("compliance_id"):
            params["Scope"].update(
                {
                    "ComplianceResourceId": module.params.get("scope").get("compliance_id"),
                }
            )
    if module.params.get("source"):
        params["Source"] = {}
        if module.params.get("source").get("owner"):
            params["Source"].update(
                {
                    "Owner": module.params.get("source").get("owner"),
                }
            )
        if module.params.get("source").get("identifier"):
            params["Source"].update(
                {
                    "SourceIdentifier": module.params.get("source").get("identifier"),
                }
            )
        if module.params.get("source").get("details"):
            params["Source"].update(
                {
                    "SourceDetails": module.params.get("source").get("details"),
                }
            )
    if module.params.get("input_parameters"):
        params["InputParameters"] = module.params.get("input_parameters")
    if module.params.get("execution_frequency"):
        params["MaximumExecutionFrequency"] = module.params.get("execution_frequency")
    params["ConfigRuleState"] = "ACTIVE"

    client = module.client("config", retry_decorator=AWSRetry.jittered_backoff())

    existing_rule = rule_exists(client, module, params)

    if state == "present":
        if not existing_rule:
            create_resource(client, module, params, result)
        else:
            update_resource(client, module, params, result)

    if state == "absent":
        if existing_rule:
            delete_resource(client, module, params, result)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: rds_cluster_param_group
version_added: 7.6.0
short_description: Manage RDS cluster parameter groups
description:
  - Creates, modifies, and deletes RDS cluster parameter groups.
options:
  state:
    description:
      - Specifies whether the RDS cluster parameter group should be present or absent.
    default: present
    choices: [ 'present' , 'absent' ]
    type: str
  name:
    description:
      - The name of the RDS cluster parameter group to create, modify or delete.
    required: true
    type: str
  description:
    description:
      - The description for the RDS cluster parameter group.
      - Required for O(state=present).
    type: str
  db_parameter_group_family:
    description:
      - The RDS cluster parameter group family name.
      - An RDS cluster parameter group can be associated with one and only one RDS cluster parameter group family,
        and can be applied only to a RDS cluster running a database engine and engine version compatible with that RDS cluster parameter group family.
      - Please use M(amazon.aws.rds_engine_versions_info) module To list all of the available parameter group families for a DB engine.
      - The RDS cluster parameter group family is immutable and can't be changed when updating a RDS cluster parameter group.
        See U(https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbclusterparametergroup.html)
      - Required for O(state=present).
    type: str
  parameters:
    description:
      - A list of parameters to update.
    type: list
    elements: dict
    suboptions:
      parameter_name:
        description: Specifies the name of the parameter.
        type: str
        required: true
      parameter_value:
        description:
        - Specifies the value of the parameter.
        type: str
        required: true
      apply_method:
        description:
        - Indicates when to apply parameter updates.
        choices:
        - immediate
        - pending-reboot
        type: str
        required: true
author:
  - "Aubin Bikouo (@abikouo)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Add or change a parameter group, in this case setting authentication_timeout to 200
  amazon.aws.rds_cluster_param_group:
      state: present
      name: test-cluster-group
      description: 'My test RDS cluster group'
      db_parameter_group_family: 'mysql5.6'
      parameters:
          - parameter_name: authentication_timeout
            parameter_value: "200"
            apply_method: immediate
      tags:
          Environment: production
          Application: parrot

- name: Remove a parameter group
  amazon.aws.rds_cluster_param_group:
      state: absent
      name: test-cluster-group
"""

RETURN = r"""
db_cluster_parameter_group:
    description: dictionary containing all the RDS cluster parameter group information.
    returned: success
    type: complex
    contains:
        db_cluster_parameter_group_arn:
            description: The Amazon Resource Name (ARN) for the RDS cluster parameter group.
            type: str
            returned: when O(state=present)
            sample: "arn:aws:rds:us-west-2:123456789012:cluster-pg:ansible-test-123456789012-redhat-cluster-param-group"
        db_cluster_parameter_group_name:
            description: The name of the RDS cluster parameter group.
            type: str
            returned: when O(state=present)
            sample: "ansible-test-123456789012-redhat-cluster-param-group"
        db_parameter_group_family:
            description: The name of the RDS parameter group family that this RDS cluster parameter group is compatible with.
            type: str
            returned: when O(state=present)
            sample: "postgres16"
        description:
            description: Provides the customer-specified description for this RDS cluster parameter group.
            type: str
            returned: when O(state=present)
            sample: "RDS cluster param group"
        tags:
            description: A dictionary of tags.
            type: dict
            returned: when O(state=present)
            sample: {
                "another": "tag",
                "resource_prefix": "ansible-test-53268383-redhat",
                "some": "tag"
            }
"""

from itertools import zip_longest
from typing import Any
from typing import Dict
from typing import List

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.rds import describe_db_cluster_parameter_groups
from ansible_collections.amazon.aws.plugins.module_utils.rds import describe_db_cluster_parameters
from ansible_collections.amazon.aws.plugins.module_utils.rds import ensure_tags
from ansible_collections.amazon.aws.plugins.module_utils.rds import get_tags
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list


def modify_parameters(
    module: AnsibleAWSModule, connection: Any, group_name: str, parameters: List[Dict[str, Any]]
) -> bool:
    current_params = describe_db_cluster_parameters(module, connection, group_name)
    parameters = snake_dict_to_camel_dict(parameters, capitalize_first=True)
    # compare current resource parameters with the value from module parameters
    changed = False
    for param in parameters:
        found = False
        for current_p in current_params:
            if param.get("ParameterName") == current_p.get("ParameterName"):
                found = True
                if not current_p["IsModifiable"]:
                    module.fail_json(f"The parameter {param.get('ParameterName')} cannot be modified")
                changed |= any((current_p.get(k) != v for k, v in param.items()))
        if not found:
            module.fail_json(msg=f"Could not find parameter with name: {param.get('ParameterName')}")
    if changed:
        if not module.check_mode:
            # When calling modify_db_cluster_parameter_group() function
            # A maximum of 20 parameters can be modified in a single request.
            # This is why we are creating chunk containing at max 20 items
            for chunk in zip_longest(*[iter(parameters)] * 20, fillvalue=None):
                non_empty_chunk = [item for item in chunk if item]
                try:
                    connection.modify_db_cluster_parameter_group(
                        aws_retry=True, DBClusterParameterGroupName=group_name, Parameters=non_empty_chunk
                    )
                except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                    module.fail_json_aws(e, msg="Couldn't update RDS cluster parameters")
    return changed


def ensure_present(module: AnsibleAWSModule, connection: Any) -> None:
    group_name = module.params["name"]
    db_parameter_group_family = module.params["db_parameter_group_family"]
    tags = module.params.get("tags")
    purge_tags = module.params.get("purge_tags")
    changed = False

    response = describe_db_cluster_parameter_groups(module=module, connection=connection, group_name=group_name)
    if not response:
        # Create RDS cluster parameter group
        params = dict(
            DBClusterParameterGroupName=group_name,
            DBParameterGroupFamily=db_parameter_group_family,
            Description=module.params["description"],
        )
        if tags:
            params["Tags"] = ansible_dict_to_boto3_tag_list(tags)
        if module.check_mode:
            module.exit_json(changed=True, msg="Would have create RDS parameter group if not in check mode.")
        try:
            response = connection.create_db_cluster_parameter_group(aws_retry=True, **params)
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't create parameter group")
    else:
        group = response[0]
        if db_parameter_group_family != group["DBParameterGroupFamily"]:
            module.warn(
                "The RDS cluster parameter group family is immutable and can't be changed when updating a RDS cluster parameter group."
            )

        if tags:
            existing_tags = get_tags(connection, module, group["DBClusterParameterGroupArn"])
            changed = ensure_tags(
                connection, module, group["DBClusterParameterGroupArn"], existing_tags, tags, purge_tags
            )

    parameters = module.params.get("parameters")
    if parameters:
        changed |= modify_parameters(module, connection, group_name, parameters)

    response = describe_db_cluster_parameter_groups(module=module, connection=connection, group_name=group_name)
    group = camel_dict_to_snake_dict(response[0])
    group["tags"] = get_tags(connection, module, group["db_cluster_parameter_group_arn"])

    module.exit_json(changed=changed, db_cluster_parameter_group=group)


def ensure_absent(module: AnsibleAWSModule, connection: Any) -> None:
    group = module.params["name"]
    response = describe_db_cluster_parameter_groups(module=module, connection=connection, group_name=group)
    if not response:
        module.exit_json(changed=False, msg="The RDS cluster parameter group does not exist.")

    if not module.check_mode:
        try:
            response = connection.delete_db_cluster_parameter_group(aws_retry=True, DBClusterParameterGroupName=group)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't delete RDS cluster parameter group")
    module.exit_json(changed=True)


def main() -> None:
    argument_spec = dict(
        state=dict(default="present", choices=["present", "absent"]),
        name=dict(required=True),
        db_parameter_group_family=dict(),
        description=dict(),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        parameters=dict(
            type="list",
            elements="dict",
            options=dict(
                parameter_name=dict(required=True),
                parameter_value=dict(required=True),
                apply_method=dict(choices=["immediate", "pending-reboot"], required=True),
            ),
        ),
    )
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[["state", "present", ["description", "db_parameter_group_family"]]],
        supports_check_mode=True,
    )

    try:
        connection = module.client("rds", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    if module.params.get("state") == "present":
        ensure_present(module=module, connection=connection)
    else:
        ensure_absent(module=module, connection=connection)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_postgresqlflexiblefirewallrule
version_added: "2.2.0"
short_description: Manage PostgreSQL flexible firewall rule instance
description:
    - Create, update and delete instance of PostgreSQL flexible firewall rule.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource. You can obtain this value from the Azure Resource Manager API or the portal.
        required: True
        type: str
    server_name:
        description:
            - The name of the server.
        required: True
        type: str
    name:
        description:
            - The name of the PostgreSQL flexible firewall rule.
        required: True
        type: str
    start_ip_address:
        description:
            - The start IP address of the PostgreSQL flexible firewall rule. Must be IPv4 format.
        type: str
    end_ip_address:
        description:
            - The end IP address of the PostgreSQL flexible firewall rule. Must be IPv4 format.
        type: str
    state:
        description:
            - Assert the state of the PostgreSQL flexible firewall rule.
            - Use C(present) to create or update a PostgreSQL flexible firewall rule and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create (or update) PostgreSQL flexible firewall rule
  azure_rm_postgresqlflexiblefirewallrule:
    resource_group: myResourceGroup
    server_name: testserver
    name: rule1
    start_ip_address: 10.0.0.16
    end_ip_address: 10.0.0.18
'''

RETURN = '''
rules:
    description:
        - A list of dictionaries containing facts for PostgreSQL Flexible Firewall Rule.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.DBforPostgreSQL/flexibleServers/flexibled9b/firewallRules/firewalld9b"
        server_name:
            description:
                - The name of the server.
            returned: always
            type: str
            sample: testserver
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: rule1
        start_ip_address:
            description:
                - The start IP address of the PostgreSQL firewall rule.
            returned: always
            type: str
            sample: 10.0.0.16
        end_ip_address:
            description:
                - The end IP address of the PostgreSQL firewall rule.
            returned: always
            type: str
            sample: 10.0.0.18
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMPostgreSqlFlexibleFirewallRules(AzureRMModuleBase):
    """Configuration class for an Azure RM PostgreSQL flexible firewall rule resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            start_ip_address=dict(
                type='str'
            ),
            end_ip_address=dict(
                type='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.server_name = None
        self.name = None
        self.start_ip_address = None
        self.end_ip_address = None

        self.results = dict(changed=False)
        self.state = None
        self.parameters = dict()

        super(AzureRMPostgreSqlFlexibleFirewallRules, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                                     supports_check_mode=True,
                                                                     supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
                if key in ['start_ip_address', 'end_ip_address']:
                    self.parameters[key] = kwargs[key]

        old_response = None
        response = None
        changed = False

        old_response = self.get_firewallrule()

        if old_response is None:
            self.log("PostgreSQL flexible firewall rule instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                changed = True
                if not self.check_mode:
                    response = self.create_update_firewallrule(self.parameters)
        else:
            self.log("PostgreSQL flexible firewall rule instance already exists")
            if self.state == 'absent':
                changed = True
                if self.check_mode:
                    response = old_response
                else:
                    response = self.delete_firewallrule()
            else:
                self.log("Need to check if PostgreSQL flexible firewall rule instance has to be deleted or may be updated")
                if (self.start_ip_address is not None) and (self.start_ip_address != old_response['start_ip_address']):
                    changed = True
                else:
                    self.parameters['start_ip_address'] = old_response['start_ip_address']
                if (self.end_ip_address is not None) and (self.end_ip_address != old_response['end_ip_address']):
                    changed = True
                else:
                    self.parameters['end_ip_address'] = old_response['end_ip_address']
                if changed:
                    if not self.check_mode:
                        response = self.create_update_firewallrule(self.parameters)
                    else:
                        response = old_response
                else:
                    response = old_response
        self.results['firewall_rule'] = response
        self.results['changed'] = changed

        return self.results

    def create_update_firewallrule(self, body):
        '''
        Creates or updates PostgreSQL flexible firewall rule with the specified configuration.

        :return: deserialized PostgreSQL flexible firewall rule instance state dictionary
        '''
        self.log("Creating / Updating the PostgreSQL flexible firewall rule instance {0}".format(self.name))

        try:
            response = self.postgresql_flexible_client.firewall_rules.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                             server_name=self.server_name,
                                                                                             firewall_rule_name=self.name,
                                                                                             parameters=body)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the PostgreSQL flexible firewall rule instance.')
            self.fail("Error creating the PostgreSQL flexible firewall rule instance: {0}".format(str(exc)))
        return self.format_item(response)

    def delete_firewallrule(self):
        '''
        Deletes specified PostgreSQL flexible firewall rule instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the PostgreSQL flexible firewall rule instance {0}".format(self.name))
        try:
            self.postgresql_flexible_client.firewall_rules.begin_delete(resource_group_name=self.resource_group,
                                                                        server_name=self.server_name,
                                                                        firewall_rule_name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the PostgreSQL flexible firewall rule instance.')
            self.fail("Error deleting the PostgreSQL flexible firewall rule instance: {0}".format(str(e)))

        return True

    def get_firewallrule(self):
        '''
        Gets the properties of the specified PostgreSQL flexible firewall rule.

        :return: deserialized PostgreSQL flexible firewall rule instance state dictionary
        '''
        self.log("Checking if the PostgreSQL flexible firewall rule instance {0} is present".format(self.name))
        try:
            response = self.postgresql_flexible_client.firewall_rules.get(resource_group_name=self.resource_group,
                                                                          server_name=self.server_name,
                                                                          firewall_rule_name=self.name)
            self.log("Response : {0}".format(response))
            self.log("PostgreSQL flexible firewall rule instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the PostgreSQL flexible firewall rule instance. Exception as {0}'.format(str(e)))
            return None
        return self.format_item(response)

    def format_item(self, item):
        d = item.as_dict()
        d = {
            'resource_group': self.resource_group,
            'id': d['id'],
            'server_name': self.server_name,
            'name': d['name'],
            'start_ip_address': d['start_ip_address'],
            'end_ip_address': d['end_ip_address']
        }
        return d


def main():
    """Main execution"""
    AzureRMPostgreSqlFlexibleFirewallRules()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Anvitha Jain (@anvjain) <anvjain@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_synce_interface_policy
short_description: Manage syncE Interface Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage syncE Interface Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Anvitha Jain (@anvjain)
options:
  template:
    description:
    - The name of the template.
    - The template must be a fabric policy template.
    type: str
    required: true
  interface_policy:
    description:
    - The name of the syncE Interface Policy.
    type: str
    aliases: [ name ]
  interface_policy_uuid:
    description:
    - The uuid of the syncE Interface Policy.
    - This parameter is required when the O(interface_policy) needs to be updated.
    type: str
    aliases: [ uuid ]
  description:
    description:
    - The description of the syncE Interface Policy.
    type: str
  admin_state:
    description:
    - The administrative state of the syncE Interface Policy.
    - The default value is disabled.
    type: str
    choices: [ enabled, disabled ]
  sync_state_msg:
    description:
    - The sync state message of the syncE Interface Policy.
    - The default value is enabled.
    type: str
    choices: [ enabled, disabled ]
  selection_input:
    description:
    - The selection input of the syncE Interface Policy.
    - The default value is disabled.
    type: str
    choices: [ enabled, disabled ]
  src_priority:
    description:
    - The source priority of the syncE Interface Policy.
    - The value must be an integer between 1 and 254.
    - The default value is 100.
    type: int
  wait_to_restore:
    description:
    - The delay before attempting to restore synchronization on a SyncE interface after a disruption.
    - The value must be an integer between 0 and 12.
    - The default value is 5.
    type: int
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new syncE interface policy
  cisco.mso.ndo_synce_interface_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    interface_policy: ansible_test_interface_policy
    admin_state: enabled
    sync_state_msg: enabled
    selection_input: enabled
    src_priority: 100
    wait_to_restore: 5
    state: present
  register: create

- name: Query a syncE interface policy with interface_policy name
  cisco.mso.ndo_synce_interface_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    interface_policy: ansible_test_interface_policy
    state: query
  register: query_one

- name: Query all syncE interface policies in the template
  cisco.mso.ndo_synce_interface_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    state: query
  register: query_all

- name: Query a syncE interface policy with interface_policy UUID
  cisco.mso.ndo_synce_interface_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    interface_policy_uuid: '{{ create.current.uuid }}'
    state: query
  register: query_one_by_uuid

- name: Delete a syncE interface policy
  cisco.mso.ndo_synce_interface_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    interface_policy: ansible_test_interface_policy
    state: absent
"""

RETURN = r"""
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.constants import ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        dict(
            template=dict(type="str", required=True),
            interface_policy=dict(type="str", aliases=["name"]),
            interface_policy_uuid=dict(type="str", aliases=["uuid"]),
            description=dict(type="str"),
            admin_state=dict(type="str", choices=["enabled", "disabled"]),
            sync_state_msg=dict(type="str", choices=["enabled", "disabled"]),
            selection_input=dict(type="str", choices=["enabled", "disabled"]),
            src_priority=dict(type="int"),
            wait_to_restore=dict(type="int"),
            state=dict(type="str", choices=["absent", "query", "present"], default="query"),
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["interface_policy"]],
            ["state", "absent", ["interface_policy"]],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    interface_policy = module.params.get("interface_policy")
    interface_policy_uuid = module.params.get("interface_policy_uuid")
    description = module.params.get("description")
    admin_state = module.params.get("admin_state")
    sync_state_msg = module.params.get("sync_state_msg")
    selection_input = module.params.get("selection_input")
    src_priority = module.params.get("src_priority")
    wait_to_restore = module.params.get("wait_to_restore")
    state = module.params.get("state")

    ops = []
    match = None

    mso_template = MSOTemplate(mso, "fabric_policy", template)
    mso_template.validate_template("fabricPolicy")

    path = "/fabricPolicyTemplate/template/syncEthIntfPolicies"
    match = get_synce_interface_policy(mso_template, interface_policy_uuid, interface_policy)

    if interface_policy_uuid or interface_policy:
        if match:
            mso.existing = mso.previous = copy.deepcopy(match.details)  # Query a specific object
    elif match:
        mso.existing = match  # Query all objects

    if state == "present":
        mso.existing = {}

        if match:
            sync_state_msg_value = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(sync_state_msg)
            selection_input_value = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(selection_input)

            if interface_policy and match.details.get("name") != interface_policy:
                ops.append(dict(op="replace", path="{0}/{1}/name".format(path, match.index), value=interface_policy))
                match.details["name"] = interface_policy

            if description is not None and match.details.get("description") != description:
                ops.append(dict(op="replace", path="{0}/{1}/description".format(path, match.index), value=description))
                match.details["description"] = description

            if admin_state and match.details.get("adminState") != admin_state:
                ops.append(dict(op="replace", path="{0}/{1}/adminState".format(path, match.index), value=admin_state))
                match.details["adminState"] = admin_state

            if sync_state_msg and match.details.get("syncStateMsgEnabled") != sync_state_msg_value:
                ops.append(dict(op="replace", path="{0}/{1}/syncStateMsgEnabled".format(path, match.index), value=sync_state_msg_value))
                match.details["syncStateMsgEnabled"] = sync_state_msg_value

            if selection_input and match.details.get("selectionInputEnabled") != selection_input_value:
                ops.append(dict(op="replace", path="{0}/{1}/selectionInputEnabled".format(path, match.index), value=selection_input_value))
                match.details["selectionInputEnabled"] = selection_input_value

            if src_priority and match.details.get("srcPriority") != src_priority:
                ops.append(dict(op="replace", path="{0}/{1}/srcPriority".format(path, match.index), value=src_priority))
                match.details["srcPriority"] = src_priority

            if wait_to_restore and match.details.get("waitToRestore") != wait_to_restore:
                ops.append(dict(op="replace", path="{0}/{1}/waitToRestore".format(path, match.index), value=wait_to_restore))
                match.details["waitToRestore"] = wait_to_restore

            mso.sanitize(match.details)

        else:
            payload = {"name": interface_policy, "templateId": mso_template.template.get("templateId"), "schemaId": mso_template.template.get("schemaId")}
            if description:
                payload["description"] = description
            if admin_state:
                payload["adminState"] = admin_state
            if sync_state_msg:
                payload["syncStateMsgEnabled"] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(sync_state_msg)
            if selection_input:
                payload["selectionInputEnabled"] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(selection_input)
            if src_priority:
                payload["srcPriority"] = src_priority
            if wait_to_restore:
                payload["waitToRestore"] = wait_to_restore

            ops.append(dict(op="add", path="{0}/-".format(path), value=copy.deepcopy(payload)))

            mso.sanitize(payload)

        mso.existing = mso.proposed

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path="{0}/{1}".format(path, match.index)))
        mso.existing = {}

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = get_synce_interface_policy(mso_template, interface_policy_uuid, interface_policy)
        if match:
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


def get_synce_interface_policy(mso_template, uuid=None, name=None, fail_module=False):
    """
    Get the SyncE Interface Policy by UUID or Name.
    :param uuid: UUID of the SyncE Interface Policy to search for -> Str
    :param name: Name of the SyncE Interface Policy to search for -> Str
    :param fail_module: When match is not found fail the ansible module -> Bool
    :return: Dict | None | List[Dict] | List[]: The processed result which could be:
              When the UUID | Name is existing in the search list -> Dict
              When the UUID | Name is not existing in the search list -> None
              When both UUID and Name are None, and the search list is not empty -> List[Dict]
              When both UUID and Name are None, and the search list is empty -> List[]
    """
    match = mso_template.template.get("fabricPolicyTemplate", {}).get("template", {}).get("syncEthIntfPolicies", [])
    if uuid or name:  # Query a specific object
        return mso_template.get_object_by_key_value_pairs(
            "SyncE Interface Policy", match, [KVPair("uuid", uuid) if uuid else KVPair("name", name)], fail_module
        )
    return match  # Query all objects


if __name__ == "__main__":
    main()



# Copyright (c) 2022, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: openssl_csr_info
short_description: Retrieve information from OpenSSL Certificate Signing Requests (CSR)
version_added: 2.10.0
author:
  - Felix Fontein (@felixfontein)
description:
  - Provided an OpenSSL Certificate Signing Requests (CSR), retrieve information.
  - This is a filter version of the M(community.crypto.openssl_csr_info) module.
options:
  _input:
    description:
      - The content of the OpenSSL CSR.
    type: string
    required: true
extends_documentation_fragment:
  - community.crypto._name_encoding
seealso:
  - module: community.crypto.openssl_csr_info
  - plugin: community.crypto.to_serial
    plugin_type: filter
"""

EXAMPLES = r"""
---
- name: Show the Subject Alt Names of the CSR
  ansible.builtin.debug:
    msg: >-
      {{
        (
          lookup('ansible.builtin.file', '/path/to/cert.csr')
          | community.crypto.openssl_csr_info
        ).subject_alt_name | join(', ')
      }}
"""

RETURN = r"""
_value:
  description:
    - Information on the certificate.
  type: dict
  contains:
    signature_valid:
      description:
        - Whether the CSR's signature is valid.
        - In case the check returns V(false), the module will fail.
      returned: success
      type: bool
    basic_constraints:
      description: Entries in the C(basic_constraints) extension, or V(none) if extension is not present.
      returned: success
      type: list
      elements: str
      sample: ['CA:TRUE', 'pathlen:1']
    basic_constraints_critical:
      description: Whether the C(basic_constraints) extension is critical.
      returned: success
      type: bool
    extended_key_usage:
      description: Entries in the C(extended_key_usage) extension, or V(none) if extension is not present.
      returned: success
      type: list
      elements: str
      sample: [Biometric Info, DVCS, Time Stamping]
    extended_key_usage_critical:
      description: Whether the C(extended_key_usage) extension is critical.
      returned: success
      type: bool
    extensions_by_oid:
      description: Returns a dictionary for every extension OID.
      returned: success
      type: dict
      contains:
        critical:
          description: Whether the extension is critical.
          returned: success
          type: bool
        value:
          description:
            - The Base64 encoded value (in DER format) of the extension.
            - B(Note) that depending on the C(cryptography) version used, it is not possible to extract the ASN.1 content
              of the extension, but only to provide the re-encoded content of the extension in case it was parsed by C(cryptography).
              This should usually result in exactly the same value, except if the original extension value was malformed.
          returned: success
          type: str
          sample: "MAMCAQU="
      sample: {"1.3.6.1.5.5.7.1.24": {"critical": false, "value": "MAMCAQU="}}
    key_usage:
      description: Entries in the C(key_usage) extension, or V(none) if extension is not present.
      returned: success
      type: str
      sample: [Key Agreement, Data Encipherment]
    key_usage_critical:
      description: Whether the C(key_usage) extension is critical.
      returned: success
      type: bool
    subject_alt_name:
      description:
        - Entries in the C(subject_alt_name) extension, or V(none) if extension is not present.
        - See O(name_encoding) for how IDNs are handled.
      returned: success
      type: list
      elements: str
      sample: ["DNS:www.ansible.com", "IP:1.2.3.4"]
    subject_alt_name_critical:
      description: Whether the C(subject_alt_name) extension is critical.
      returned: success
      type: bool
    ocsp_must_staple:
      description: V(true) if the OCSP Must Staple extension is present, V(none) otherwise.
      returned: success
      type: bool
    ocsp_must_staple_critical:
      description: Whether the C(ocsp_must_staple) extension is critical.
      returned: success
      type: bool
    name_constraints_permitted:
      description: List of permitted subtrees to sign certificates for.
      returned: success
      type: list
      elements: str
      sample: ['email:.somedomain.com']
    name_constraints_excluded:
      description:
        - List of excluded subtrees the CA cannot sign certificates for.
        - Is V(none) if extension is not present.
        - See O(name_encoding) for how IDNs are handled.
      returned: success
      type: list
      elements: str
      sample: ['email:.com']
    name_constraints_critical:
      description:
        - Whether the C(name_constraints) extension is critical.
        - Is V(none) if extension is not present.
      returned: success
      type: bool
    subject:
      description:
        - The CSR's subject as a dictionary.
        - Note that for repeated values, only the last one will be returned.
      returned: success
      type: dict
      sample: {"commonName": "www.example.com", "emailAddress": "test@example.com"}
    subject_ordered:
      description: The CSR's subject as an ordered list of tuples.
      returned: success
      type: list
      elements: list
      sample: [["commonName", "www.example.com"], ["emailAddress": "test@example.com"]]
    public_key:
      description: CSR's public key in PEM format.
      returned: success
      type: str
      sample: "-----BEGIN PUBLIC KEY-----\nMIICIjANBgkqhkiG9w0BAQEFAAOCAg8A..."
    public_key_type:
      description:
        - The CSR's public key's type.
        - One of V(RSA), V(DSA), V(ECC), V(Ed25519), V(X25519), V(Ed448), or V(X448).
        - Will start with C(unknown) if the key type cannot be determined.
      returned: success
      type: str
      sample: RSA
    public_key_data:
      description:
        - Public key data. Depends on the public key's type.
      returned: success
      type: dict
      contains:
        size:
          description:
            - Bit size of modulus (RSA) or prime number (DSA).
          type: int
          returned: When RV(_value.public_key_type=RSA) or RV(_value.public_key_type=DSA)
        modulus:
          description:
            - The RSA key's modulus.
          type: int
          returned: When RV(_value.public_key_type=RSA)
        exponent:
          description:
            - The RSA key's public exponent.
          type: int
          returned: When RV(_value.public_key_type=RSA)
        p:
          description:
            - The C(p) value for DSA.
            - This is the prime modulus upon which arithmetic takes place.
          type: int
          returned: When RV(_value.public_key_type=DSA)
        q:
          description:
            - The C(q) value for DSA.
            - This is a prime that divides C(p - 1), and at the same time the order of the subgroup of the multiplicative
              group of the prime field used.
          type: int
          returned: When RV(_value.public_key_type=DSA)
        g:
          description:
            - The C(g) value for DSA.
            - This is the element spanning the subgroup of the multiplicative group of the prime field used.
          type: int
          returned: When RV(_value.public_key_type=DSA)
        curve:
          description:
            - The curve's name for ECC.
          type: str
          returned: When RV(_value.public_key_type=ECC)
        exponent_size:
          description:
            - The maximum number of bits of a private key. This is basically the bit size of the subgroup used.
          type: int
          returned: When RV(_value.public_key_type=ECC)
        x:
          description:
            - The C(x) coordinate for the public point on the elliptic curve.
          type: int
          returned: When RV(_value.public_key_type=ECC)
        y:
          description:
            - For RV(_value.public_key_type=ECC), this is the C(y) coordinate for the public point on the elliptic curve.
            - For RV(_value.public_key_type=DSA), this is the publicly known group element whose discrete logarithm with respect
              to C(g) is the private key.
          type: int
          returned: When RV(_value.public_key_type=DSA) or RV(_value.public_key_type=ECC)
    public_key_fingerprints:
      description:
        - Fingerprints of CSR's public key.
        - For every hash algorithm available, the fingerprint is computed.
      returned: success
      type: dict
      sample: "{'sha256': 'd4:b3:aa:6d:c8:04:ce:4e:ba:f6:29:4d:92:a3:94:b0:c2:ff:bd:bf:33:63:11:43:34:0f:51:b0:95:09:2f:63',
        'sha512': 'f7:07:4a:f0:b0:f0:e6:8b:95:5f:f9:e6:61:0a:32:68:f1..."
    subject_key_identifier:
      description:
        - The CSR's subject key identifier.
        - The identifier is returned in hexadecimal, with V(:) used to separate bytes.
        - Is V(none) if the C(SubjectKeyIdentifier) extension is not present.
      returned: success
      type: str
      sample: '00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:00:11:22:33'
    authority_key_identifier:
      description:
        - The CSR's authority key identifier.
        - The identifier is returned in hexadecimal, with V(:) used to separate bytes.
        - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
      returned: success
      type: str
      sample: '00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:00:11:22:33'
    authority_cert_issuer:
      description:
        - The CSR's authority cert issuer as a list of general names.
        - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
        - See O(name_encoding) for how IDNs are handled.
      returned: success
      type: list
      elements: str
      sample: ["DNS:www.ansible.com", "IP:1.2.3.4"]
    authority_cert_serial_number:
      description:
        - The CSR's authority cert serial number.
        - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
        - This return value is an B(integer). If you need the serial numbers as a colon-separated hex string, such as C(11:22:33),
          you need to convert it to that form with P(community.crypto.to_serial#filter).
      returned: success
      type: int
      sample: 12345
"""

import typing as t
from collections.abc import Callable

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_bytes, to_text

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.csr_info import (
    get_csr_info,
)
from ansible_collections.community.crypto.plugins.plugin_utils._filter_module import (
    FilterModuleMock,
)


def openssl_csr_info_filter(
    data: str | bytes, name_encoding: t.Literal["ignore", "idna", "unicode"] = "ignore"
) -> dict[str, t.Any]:
    """Extract information from X.509 PEM certificate."""
    if not isinstance(data, (str, bytes)):
        raise AnsibleFilterError(
            f"The community.crypto.openssl_csr_info input must be a text type, not {type(data)}"
        )
    if not isinstance(name_encoding, (str, bytes)):
        raise AnsibleFilterError(
            f"The name_encoding option must be of a text type, not {type(name_encoding)}"
        )
    name_encoding = t.cast(
        t.Literal["ignore", "idna", "unicode"], to_text(name_encoding)
    )
    if name_encoding not in ("ignore", "idna", "unicode"):
        raise AnsibleFilterError(
            f'The name_encoding option must be one of the values "ignore", "idna", or "unicode", not "{name_encoding}"'
        )

    module = FilterModuleMock({"name_encoding": name_encoding})
    try:
        return get_csr_info(
            module=module, content=to_bytes(data), validate_signature=True
        )
    except OpenSSLObjectError as exc:
        raise AnsibleFilterError(str(exc)) from exc


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "openssl_csr_info": openssl_csr_info_filter,
        }
